/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 Dump the argument given file(s) to the standard output

 Build: make a_cat
*/
#include "../src/wybyteflow.h"
#include "../src/wy_uty.h"
#include <termios.h>

// [Syn] Read data from inf untill EOF, and write to outf.
//
// [Ret] OK
//       ....
//
static WyRet cat_file(WyByteFlow& outf, WyByteFlow& inf)
{
 const size_t Btr(1024);   // bytes to read
 WyRet r;
 for(;;) {
   size_t n_rd;
   size_t n_wr;
   WyStr buf;

   if((r=inf.read(buf,Btr,n_rd))!=OK) {
     WY_RETURN(r);
   }
   if(n_rd==0) {
     break;
   }
   if((r=outf.write(buf,n_wr))!=OK) {
     WY_RETURN(r);
   }
   if(n_rd!=n_wr) {
     WY_RETURN(Wym_EIO);
   }
 }
 return(OK);
};

// Command-Line Message
//
static char *cmd_syntax=
   "\n"
   "Dump the argument given file(s) to the standard output\n"
   "\n"
   " [Usage]$a_cat {file}\n"
   "\n"
   ;

int main(int argc, char *argv[])
try {

 // Command-line options
 //
 {
  const char optstr[]="h";
  int optch;
  
  while((optch=::getopt(argc,argv,optstr))!=-1) {
    switch(optch) {
      case 'h':
	      Wy::cout << cmd_syntax;
	      return(0);
      case ':':     // missing parameter
      case '?':     // unknown option
	      Wy::cerr << "parameter error\n";
	      Wy::cerr << cmd_syntax;
              return(-1);
      default:
	      Wy::cerr << "parameter fault\n";
              return(-1);
    }
  }
 }

 WyRet r;

 WyFileStat ostt;
 if((r=Wy::cout.stat(ostt))!=OK) {
   WY_THROW(r);
 }
 if(ostt.is_blk()) {
   Wy::cerr << "output file is a block device\n";
   return(-1);
 }
 for( ; optind<argc; optind++) {
   const WyStr fname(argv[optind]);
   WyByteFlow infile(fname,O_RDONLY);
   WyFileStat istt;
   if((r=infile.stat(istt))!=OK) {
     WY_THROW(r);
   }
   if((istt.dev()==ostt.dev())&&(istt.ino()==ostt.ino())) {
     Wy::cerr << "input file(" << fname << ") is identical to output file\n";
     return(-1);
   }
   if(istt.is_blk()) {
     Wy::cerr << "input file(" << fname << ") is a block device\n";
     return(-1);
   }
   if((r=cat_file(Wy::cout, infile))!=OK) {
     Wy::cerr << "input file(" << fname << "): ";
     WY_THROW(r);
   }
 }
 if(Wy::cout.ftype()==S_IFREG) {
   Wy::cout._fdatasync();
 } else if(Wy::cout._isatty()) {
   Wy::cout._tcdrain();
 } else {};

 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
