/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2006

   Simulate popen

   Build: make a_popen  (g++ a_popen.cpp -pthread -lwy)
*/

#include "../src/wy_uty.h"
#include "../src/wyfifofile.h"
#include <sys/types.h>          // fork
#include <unistd.h>             // fork

// Create a pipe and a process running command loaded by shell, reset
// ff to the read or write end of the pipe according to type.
// Functionality is similar to ::popen
//
static WyRet a_popen(WyFifoFile& ff, const char* command, const char* type)
{
 WyRet r;
 pid_t child_pid;
 WyFifoFile tmpff;

 if(type==NULL) {
   WY_RETURN( Wym_EFAULT );
 }
 if(::strlen(type)!=1) {
   WY_RETURN( Wym_EINVAL);
 }

 // Create a pipe and reset ff/tmpff to the ends of the pipe according
 // to type(argument) (can also be done by using swap)
 //
 switch(type[0]) {
   case 'r':
            if((r=WyFifoFile::mkpipe(ff,tmpff))!=OK) {
              WY_RETURN(r);
            }
            break;
   case 'w':
            if((r=WyFifoFile::mkpipe(tmpff,ff))!=OK) {
              WY_RETURN(r);
            }
            break;
   default:
            WY_RETURN( Wym_EINVAL );
 };

 child_pid=::fork();
 if(child_pid==-1) {
   WY_THROW( WyRet() );
 }

 if(child_pid==0) {

   // Child process: 
   //
   // Connecting stand input or output to the pipe and
   // invoke shell to load and run command(argument)
   //
   ff.reset();
   if(type[0]=='r') {
     Wy::cout.reset(tmpff);
   } else {
     Wy::cin.reset(tmpff);
   }
   ::execl("/bin/sh", "sh", "-c", command, NULL);
   WY_RETURN( WyReply(errno) );
 }

 // Parent process: 
 //
 // Clear the unused end
 //
 tmpff.reset();
 return(OK);
};

int main(void)
try {
 WyRet r;
 WyFifoFile ff;

 // Test write
 if((r=a_popen(ff, "cat", "w"))!=OK) {
   throw(r);
 }
 ff << "Hello, this is written to a_popen\n";

 // Test read
 if((r=a_popen(ff, "ls -l -S", "r"))!=OK) {
   throw(r);
 }

 WyStr str;
 str._reserve(1024);

 for(;;) {
   ff >> str;
   if(str.size()==0) {
     break;
   }
   Wy::cout << str;
 };

 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
