/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

  Simulate shell command who

  Build: make a_who
*/

#include "../src/wy_uty.h"
#include "../src/wyregfile.h"
#include "../src/wydate.h"
#include <utmp.h>

WyRet dump_info(struct utmp& ubuf)
try {
 if(ubuf.ut_type!=USER_PROCESS) {
   return(OK);
 }
 WyStr str;
 str=ubuf.ut_user;
 str.resize(8,' '); str+=' ';  
 str+=ubuf.ut_line;
 str.resize(14,' '); str+=' ';
 str+=ubuf.ut_host;
 str.resize(20,' '); str+=' ';

 WyTimeSpec utm(ubuf.ut_tv.tv_sec,ubuf.ut_tv.tv_usec*1000);
 str+=Wy::wrd( WyDate(Wy::local_tzname(),utm) );
 
 Wy::cout << str << '\n';
 return(OK);
}
catch(const WyStr::Reply& e) {
 WY_RETURN(e);
}
catch(const WyByteFlow::Reply& e) {
 WY_RETURN(e);
}
catch(const WyRet& e) {
 throw;
}
catch(...) {
 WY_TERMINATE("unknown throw type");
};

int main(void)
try {
 WyRet r;
 size_t n_rd;
 struct utmp ubuf;
 WyRegFile ufile(UTMP_FILE,O_RDONLY);

 for(;;) {
   if((r=ufile._xread(&ubuf,sizeof(ubuf),n_rd))!=OK) {
     WY_THROW(r);
   }
   if(n_rd!=sizeof(ubuf)) {
     if(n_rd==0) {
       break;
     }
     WY_THROW( WyRet(Wym_EIO) );
   }
   if((r=dump_info(ubuf))!=OK) {
     throw(r);
   }
 };
 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
