/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Build: make demo_thread
*/
#include "../src/wy_thread.h"
#include "../src/wytimespec.h"
#include "../src/wy_uty.h"

// The demo thread class
//
class MyThread : public Wy_Thread {
  public:
    MyThread() {};
    ~MyThread() throw()
        {
         cancel();       // stop possibly running thread before others
         wait_not(Active);
        };
  protected:

    // Reimplement tmain() to run in the created thread
    //
    WyRet tmain(void) {
       Wy::cout << "tmain(): ";
       const WyTimeSpec DelayTime(0,500000000);   // 0.5 sec
       for(;;) {
         Wy::sleep(DelayTime); // delay and cancel point
         Wy::cout << 'Z';      // print 'Z' in the endless loop
       }
       return(OK);
    };
};

int main(void)
try {
 WyRet r;
 MyThread thrd;
 
 // Begin the thread
 if((r=thrd.begin())!=OK) {
   WY_THROW(r);
 }

 // Sleep four seconds for the thread
 Wy::sleep(WyTimeSpec(4,Wy_Nano()));

 // Cancel the thread (signal sent, the thread may not terminated yet)
 if((r=thrd.cancel())!=OK) {
   WY_THROW(r);
 }

 // Wait until the thread terminated
 if((r=thrd.wait_not(Wy_Thread::Active))!=OK) {
   WY_THROW(r);
 }

 // Get the exit code of the thread. 
 r=thrd.exitcode(); WY_HERE(r); // Make it says from here
 if(r!=Wym_ETHRDCAN) {
   WY_THROW(r);
 }

 // Print the results.
 Wy::cout << "\nmain() report: canceled that 'Z'ing thread, it exited :"
          << Wy::wrd(r) << '\n';
 return(0);
}
catch(const WyRet& e) {
 if(e!=OK) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->c_repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 return(-1);
};
