/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

   Build: chk_hstr
*/

#include "wy__hstr.h"
#include <iostream>

#if WY__HSTR_VERSION!=30
#error Test code is for WY__HSTR_VERSION 30
#endif

void chk_invariant(const Wy__HStr* str)
{
 if(str->data()==0) {
   WY_THROW( WyRet() );
 }
 if(str->c_str()!=str->data()) {
   WY_THROW( WyRet() );
 }
 if(str->data()[ str->size() ]!=0) {
   WY_THROW( WyRet() );
 }
};

bool chk_default(const Wy__HStr* ptr)
{
 chk_invariant(ptr);
 if(ptr->size()!=0) {
   return(false);
 }
 if(ptr->c_str()[0]!=0) {
   return(false);
 }
 if(ptr->_capacity()!=Wy__HStr::min_capacity()) {
   return(false);
 }
 return(true);
};

#define CHK_DEFAULT(expr) if(chk_default(expr)==false) { WY_THROW(WyRet() ); };
#define CHK_NOT_DEFAULT(expr) if(chk_not_default(expr)==false) { WY_THROW(WyRet() ); };

static void ck_str(void)
{
 const char Ref[]="0123456789ab";
 Wy__HStr* tp0= Wy__HStr::alloc();
 Wy__HStr* tp1= Wy__HStr::alloc(100);
 Wy__HStr* tp2= Wy__HStr::alloc(WyCSeg(Ref,3));
 Wy__HStr* tp3= Wy__HStr::alloc(WyCSeg(Ref+1,3));
 Wy__HStr* tp4= Wy__HStr::alloc(WyCSeg(Ref+2,3),WyCSeg(Ref+5,3));
 Wy__HStr* tp5= Wy__HStr::alloc(WyCSeg(Ref+3,3),WyCSeg(Ref+6,3),WyCSeg(Ref+9,3));
 Wy__HStr* tmp= Wy__HStr::alloc();
 WyRet r;

 chk_invariant(tp0);
 chk_invariant(tp1);
 chk_invariant(tp2);
 chk_invariant(tp3);
 chk_invariant(tp4);
 chk_invariant(tp5);
 chk_invariant(tmp);
 try {
   // chk alloc tp*
   CHK_DEFAULT(tp0);
   if(tp1->c_str()!=tp1->data()) {
     WY_THROW( WyRet() );
   }
   if((tp1->c_str()[0]!=0)||(tp1->size()!=0)) {
     WY_THROW( WyRet() );
   }
   if(tp1->_capacity()<100) {
     WY_THROW( WyRet() );
   }
   if(::memcmp(Ref,tp2->c_str(),tp2->size())!=0) {
     WY_THROW( WyRet() );
   }
   if(::memcmp(Ref+1,tp3->c_str(),tp3->size())!=0) {
     WY_THROW( WyRet() );
   }
   if(::memcmp(Ref+2,tp4->c_str(),tp4->size())!=0) {
     WY_THROW( WyRet() );
   }
   if(::memcmp(Ref+3,tp5->c_str(),tp5->size())!=0) {
     WY_THROW( WyRet() );
   }
   if(tp2->c_str()[tp2->size()]!=0) {
     WY_THROW( WyRet() );
   }
   if(tp3->c_str()[tp3->size()]!=0) {
     WY_THROW( WyRet() );
   }
   if(tp4->c_str()[tp4->size()]!=0) {
     WY_THROW( WyRet() );
   }
   if(tp5->c_str()[tp5->size()]!=0) {
     WY_THROW( WyRet() );
   }
 
   // alloc(size_t,char)
   {
     Wy__HStr* tt= Wy__HStr::alloc(4,'u');
     chk_invariant(tt);
     if((tt->size()!=4)||(Wy::_strcmp(tt->c_str(),"uuuu")!=0)||(tt->_capacity()<4)) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // alloc(const Wy__HStr&)
   {
     Wy__HStr* tt= Wy__HStr::alloc(*tp2);
     chk_invariant(tt);
     if(Wy::_strcmp(tt->cseg(),tp2->cseg())!=0) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // alloc(const WCSeg&,char)
   {
     Wy__HStr* tt= Wy__HStr::alloc(WyCSeg("abc",3),'z');
     chk_invariant(tt);
     if(Wy::_strcmp(tt->cseg(),"abcz")!=0) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // alloc(char,const WCSeg&)
   {
     Wy__HStr* tt= Wy__HStr::alloc('z',WyCSeg("abc",3));
     chk_invariant(tt);
     if(Wy::_strcmp(tt->cseg(),"zabc")!=0) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // alloc(const WCSeg&,size_t,char)
   {
     Wy__HStr* tt= Wy__HStr::alloc(WyCSeg("abc",3),2,'z');
     chk_invariant(tt);
     if(Wy::_strcmp(tt->cseg(),"abczz")!=0) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // alloc(const WCSeg&,size_t,char,const WyCSeg&)
   {
     Wy__HStr* tt= Wy__HStr::alloc(WyCSeg("aa"),2,'b',WyCSeg("cc"));
     chk_invariant(tt);
     if(Wy::_strcmp(tt->cseg(),"aabbcc")!=0) {
       Wy__HStr::free(tt);
       WY_THROW( WyRet() );
     }
     Wy__HStr::free(tt);
   }

   // cseg
   if(Wy::_strcmp(tp2->cseg(),WyCSeg(Ref,3))!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(tp2->cseg(1),WyCSeg(Ref+1,2))!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(tp2->cseg(1,2),WyCSeg(Ref+1,2))!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(tp2->cseg(1,3),WyCSeg(Ref+1,2))!=0) {
     WY_THROW( WyRet() );
   }
   chk_invariant(tp2);
  
   // operator[]
   if((tp3->_at(0)!='1')||(tp3->_at(1)!='2')) {
     WY_THROW( WyRet() );
   }
   chk_invariant(tp3);

   // set_string()
   {
     chk_invariant(tp0);
     tp0->set_string();
     CHK_DEFAULT(tp0);

     tmp->set_string();
     CHK_DEFAULT(tmp);

     if((r=tmp->set_string(*tp3))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),tp3->cseg())!=0) {
       WY_THROW( WyRet() );
     }
     if(tmp->c_str()[tmp->size()]!=0) {
       WY_THROW( WyRet() );
     }

     tmp->set_string();
     CHK_DEFAULT(tmp);
     if((r=tmp->set_string(tp3->cseg()))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(tmp->cseg(),tp3->cseg())!=0) {
       WY_THROW( WyRet() );
     }
     if(tmp->c_str()[tmp->size()]!=0) {
       WY_THROW( WyRet() );
     }

     tmp->set_string();
     CHK_DEFAULT(tmp);
     if((r=tmp->set_string(0,'x'))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(tmp->cseg(),"")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->set_string(2,'x'))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(tmp->cseg(),"xx")!=0) {
       WY_THROW( WyRet() );
     }
     if(tmp->c_str()[tmp->size()]!=0) {
       WY_THROW( WyRet() );
     }
   }

   // append
   {
     tmp->set_string();
     CHK_DEFAULT(tmp);

     if((r=tmp->append(*tp2))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"012")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->append(tp2->cseg(1)))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"01212")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->append(0,'a'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"01212")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->append(1,'a'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"01212a")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->append('b'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"01212ab")!=0) {
       WY_THROW( WyRet() );
     }
   }

   // find 
   {
     size_t fidx;
     tmp->set_string("HELOWOELD");
     chk_invariant(tmp);
     if(tmp->find(0,'X')==true) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(tmp->find(&fidx,'L')==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=2) {
       WY_THROW( WyRet() );
     }

     if(tmp->rfind(0,'X')==true) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(tmp->rfind(&fidx,'L')==false) {
       WY_THROW( WyRet() );
     }
     if(fidx!=7) {
       WY_THROW( WyRet() );
     }

     if(tmp->find(&fidx,1,'L')==false) {
       WY_THROW( WyRet() );
     }
     if(fidx!=2) {
       WY_THROW( WyRet() );
     }
     if(tmp->find(&fidx,3,'L')==false) {
       WY_THROW( WyRet() );
     }
     if(fidx!=7) {
       WY_THROW( WyRet() );
     }
     if(tmp->find(&fidx,3,'x')==true) {
       WY_THROW( WyRet() );
     }
     if(fidx!=7) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);

     if(tmp->rfind(&fidx,3,'L')==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=2) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,8,'L')==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=7) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,8,'x')==true) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=7) {
       WY_THROW( WyRet() );
     }

     if(tmp->find(&fidx,0,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }
     /*fidx=100;
     if(tmp->find(&fidx,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }*/
     if(tmp->find(&fidx,2,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=6) {
       WY_THROW( WyRet() );
     }
     fidx=100;
     if(tmp->rfind(&fidx,10,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=6) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,tmp->size()-1,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=6) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,5,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }
     fidx=100;
     if(tmp->rfind(&fidx,3,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }
     fidx=100;
     if(tmp->rfind(&fidx,1,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,0,WyCSeg("EL"))==true) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=1) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind(&fidx,WyCSeg("EL"))==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=6) {
       WY_THROW( WyRet() );
     }
   }

   // find_not
   {
     size_t fidx=100;
     tmp->set_string("xxxOKbbb");
     chk_invariant(tmp);
     if(tmp->find_not(&fidx,0,'x')==false) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
     if(fidx!=3) {
       WY_THROW( WyRet() );
     }
     if(tmp->find_not(&fidx,5,'b')==true) {
       WY_THROW( WyRet() );
     }
     if(fidx!=3) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind_not(&fidx,tmp->size(),'b')==false) {
       WY_THROW( WyRet() );
     }
     if(fidx!=4) {
       WY_THROW( WyRet() );
     }
     if(tmp->rfind_not(&fidx,2,'x')==true) {
       WY_THROW( WyRet() );
     }
     if(fidx!=4) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
   }

   // insert
   {
     chk_invariant(tmp);
     tmp->set_string();
     chk_invariant(tmp);
     if((r=tmp->insert(0,WyCSeg("fg")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"fg")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->insert(1,WyCSeg("xz")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"fxzg")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->insert(tmp->size(),WyCSeg("5")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"fxzg5")!=0) {
       WY_THROW( WyRet() );
     }

     tmp->set_string();
     if((r=tmp->insert(0,2,'a'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"aa")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->insert(1,3,'b'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"abbba")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->insert(tmp->size(),1,'c'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"abbbac")!=0) {
       WY_THROW( WyRet() );
     }

     if((r=tmp->insert(1,*tp3))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(tmp->cseg(),"a123bbbac")!=0) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
   }

   // erase
   {
     tmp->set_string();
     chk_invariant(tmp);
     if((r=tmp->erase(0))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"")!=0) {
       WY_THROW( WyRet() );
     }
     tmp->set_string("987654321");
     if((r=tmp->erase(4))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(tmp->cseg(),"9876")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->erase(2,1))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"986")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->erase(2,10))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"98")!=0) {
       WY_THROW( WyRet() );
     }
     chk_invariant(tmp);
   }

   // resize
   {
     tmp->set_string("ggff");
     chk_invariant(tmp);
     if((r=tmp->resize(2,'t'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"gg")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->resize(4,'s'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"ggss")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->resize(3))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"ggs")!=0) {
       WY_THROW( WyRet() );
     }
     {
       const char RES[]={'g','g','s',0,0,0};
       if((r=tmp->resize(6))!=OK) {
         WY_THROW(r);
       }
       chk_invariant(tmp);
       if(Wy::_strcmp(tmp->cseg(),WyCSeg(RES,6))!=0) {
         WY_THROW( WyRet() );
       }
     }
   }

   // replace
   {
     chk_invariant(tmp);
     tmp->set_string();
     chk_invariant(tmp);
     if((r=tmp->replace(0,0,WyCSeg("kio")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"kio")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->replace(1,0,WyCSeg("kio")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"kkioio")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->replace(2,3,WyCSeg("x")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"kkxo")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->replace(2,3,WyCSeg("x")))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"kkx")!=0) {
       WY_THROW( WyRet() );
     }

     if((r=tmp->replace(2,3,2,'8'))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"kk88")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tmp->replace(1,2,*tp3))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if(Wy::_strcmp(tmp->cseg(),"k1238")!=0) {
       WY_THROW( WyRet() );
     }
   }

   // strcmp
   {
     chk_invariant(tmp);
     tmp->set_string();
     chk_invariant(tmp);
     if(tmp->strcmp(*tp0)!=0) {
       WY_THROW( WyRet() );
     }
     if(tp3->strcmp(*tp3)!=0) {
       WY_THROW( WyRet() );
     }
     if(tp3->strcmp(*tp4)==0) {
       WY_THROW( WyRet() );
     }
     if(tp3->strcmp(tp3->cseg())!=0) {
       WY_THROW( WyRet() );
     }
     if(tp3->strcmp(tp4->cseg())==0) {
       WY_THROW( WyRet() );
     }
   }

   // copyto
   {
     char buf[16];
     size_t rlen=100;
     chk_invariant(tp3);
     if((r=tp3->copyto(&rlen,buf,sizeof(buf),1))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tp3);
     if(rlen!=2) {
       WY_THROW( WyRet() );
     }
     if(Wy::_strcmp(WyCSeg(buf,rlen),"23")!=0) {
       WY_THROW( WyRet() );
     }
     if((r=tp3->copyto(&rlen,buf,0,1))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tp3);
     if(rlen!=0) {
       WY_THROW( WyRet() );
     }
     if(Wy::_strcmp(WyCSeg(buf,rlen),"23")==0) {
       WY_THROW( WyRet() );
     }
     if((r=tp3->copyto(&rlen,buf+1,sizeof(buf)-1,0))!=OK) {
       WY_THROW(r);
     }
     if(Wy::_strcmp(WyCSeg(buf,4),"2123")!=0) {
       WY_THROW( WyRet() );
     }
     if(rlen!=3) {
       WY_THROW( WyRet() );
     }
   }

    // _setsize(size_t)
    {
     if((r=tmp->set_string("helo-world"))!=OK) {
       WY_THROW(r);
     }
     chk_invariant(tmp);
     if((r=tmp->_setsize(3))!=OK) {
       WY_THROW(r);
     }
     if(tmp->strcmp("hel")!=0) {
       WY_THROW(r);
     }
     if((r=tmp->_setsize(tmp->_capacity()))!=OK) {
       WY_THROW(r);
     }
     if((r=tmp->_setsize(tmp->_capacity()+1))!=Wym_EFBIG) {
       WY_THROW(r);
     }
    }
   }
   catch(...) {
     Wy__HStr::free(tp0);
     Wy__HStr::free(tp1);
     Wy__HStr::free(tp2);
     Wy__HStr::free(tp3);
     Wy__HStr::free(tp4);
     Wy__HStr::free(tp5);
     Wy__HStr::free(tmp);
     throw;
   };
  
   Wy__HStr::free(tp0);
   Wy__HStr::free(tp1);
   Wy__HStr::free(tp2);
   Wy__HStr::free(tp3);
   Wy__HStr::free(tp4);
   Wy__HStr::free(tp5);
   Wy__HStr::free(tmp);
};
  
static void ck_error(void)
{
 Wy__HStr* GPtr0=Wy__HStr::alloc();     // GPtr0 is common for the check
 Wy__HStr* GPtr1=Wy__HStr::alloc("1");  // GPtr1 is common for the check
 WyRet r;

 try {
  
   // alloc(size_t)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(Wy__HStr::max_capacity()+1);
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // alloc(size_t,char)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(size_t(-1),'x');
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // alloc(const WyCSeg&)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(WyCSeg("",size_t(-1)));
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // alloc(const WyCSeg&,const WyCSeg&)
   {
     try {
       WyCSeg bseg("",size_t(-1));
       Wy__HStr* hptr=Wy__HStr::alloc(bseg,bseg);
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // alloc(const WyCSeg&,const WyCSeg&,const WyCSeg&)
   {
     try {
       WyCSeg bseg("",size_t(-1));
       Wy__HStr* hptr=Wy__HStr::alloc(bseg,bseg,bseg);
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   // alloc(const Wy__HStr&)
   {
   }

   // alloc(const WCSeg&,char)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(WyCSeg("",Wy__HStr::max_capacity()),'z');
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // alloc(const WCSeg&,size_t,char)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(WyCSeg("",Wy__HStr::max_capacity()),1,'z');
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   // alloc(const WCSeg&,size_t,char,const WyCSeg&)
   {
     try {
       Wy__HStr* hptr=Wy__HStr::alloc(WyCSeg(""),Wy__HStr::max_capacity(),'z',
                                      WyCSeg("a"));
       Wy__HStr::free(hptr);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EFBIG) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   // cseg(size_t)
   {
     try {
       GPtr0->cseg(GPtr0->size()+1);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EINVAL) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   // cseg(size_t,size_t)
   {
     try {
       GPtr0->cseg(GPtr0->size()+1,2);
       WY_THROW( WyRet() );
     }
     catch(const Wy__HStr::Reply& e) {
       if(e!=Wym_EINVAL) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   // set_string(..)
   if((r=GPtr0->set_string(*GPtr0))!=OK) {  // note GPtr.size()==0, so OK
     WY_THROW(r);
   }
   if((r=GPtr1->set_string(*GPtr1))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=GPtr1->set_string(GPtr1->cseg()))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=GPtr1->set_string(WyCSeg("",size_t(-1))))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   if((r=GPtr1->set_string(size_t(-1),'a'))!=Wym_EFBIG) {
     WY_THROW(r);
   }

   // append
   if((r=GPtr1->append(*GPtr1))!=OK) {
     // imp. feature
     WY_THROW(r);
   }
   if((r=GPtr1->append(GPtr1->cseg()))!=OK) {
     // imp. feature
     WY_THROW(r);
   }
   if((r=GPtr1->append(WyCSeg("",size_t(-1))))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   if((r=GPtr1->append(size_t(-1),'a'))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   if((r=GPtr1->set_string(GPtr1->_capacity(),'F'))!=OK) {
     WY_THROW(r);
   }
   if((r=GPtr1->append('x'))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   
   // insert
   if((r=GPtr1->set_string(3,'F'))!=OK) {
     WY_THROW(r);
   }
   if((r=GPtr1->insert(GPtr1->size()+1,WyCSeg("x")))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->insert(1,GPtr1->cseg()))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=GPtr1->insert(1,WyCSeg("",GPtr1->_capacity())))!=Wym_EFBIG) {
     WY_THROW(r);
   }

   if((r=GPtr1->insert(GPtr1->size()+1,1,'a'))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->insert(1,GPtr1->_capacity(),'a'))!=Wym_EFBIG) {
     WY_THROW(r);
   }

   if((r=GPtr1->insert(GPtr1->size()+1,*GPtr0))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->insert(0,*GPtr1))!=Wym_ELOOP) {
     WY_THROW(r);
   }

   // erase
   if((r=GPtr1->erase(GPtr1->size()+1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->erase(GPtr1->size()+1,1))!=Wym_EINVAL) {
     WY_THROW(r);
   }

   // resize
   if((r=GPtr1->resize(GPtr1->_capacity()+1))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   if((r=GPtr1->resize(GPtr1->_capacity()))!=OK) {
     WY_THROW(r);
   }
   if((r=GPtr1->resize(GPtr1->_capacity()+1,'a'))!=Wym_EFBIG) {
     WY_THROW(r);
   }

   // replace
   if((r=GPtr1->set_string(3,'F'))!=OK) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(GPtr1->size()+1,1,*GPtr1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(1,1,*GPtr1))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(GPtr1->size()+1,1,WyCSeg("",1)))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(1,1,WyCSeg("",GPtr1->_capacity())))!=Wym_EFBIG) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(1,1,GPtr1->cseg()))!=Wym_ELOOP) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(GPtr1->size()+1,1,1,'a'))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if((r=GPtr1->replace(1,1,GPtr1->_capacity(),'a'))!=Wym_EFBIG) {
     WY_THROW(r);
   }

   // copyto
   {
    char buf[16];
    size_t rlen=99;
    if((r=GPtr1->set_string(3,'F'))!=OK) {
      WY_THROW(r);
    }
    if((r=GPtr1->copyto(&rlen,(char*)0,1,0))!=Wym_EFAULT) {
      WY_THROW(r);
    }
    if((r=GPtr1->copyto(&rlen,buf,sizeof(buf),GPtr1->size()+1))!=Wym_EINVAL) {
      WY_THROW(r);
    }
   }
  }
  catch(...) {
    Wy__HStr::free(GPtr0);
    Wy__HStr::free(GPtr1);
    throw;
  };
};

static void print_ret(const WyRet& r)
{
#ifndef WYLIB_CONFIG_NORSLI
 if(r.sli_ptr()==0) {
   std::cerr << r->c_repstr();
 } else {
   std::cerr << r.sli_ptr()->src_file()<< '(' << r.sli_ptr()->src_line()
             << "): " << r->c_repstr();
 }
#else
   std::cerr << r->c_repstr();
#endif
};

static const std::string chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking wy__hstr.h ...\n";
 if(std::string(Wy__HStr::class_name)!="Wy__HStr") {
   WY_THROW(WyRet() );
 }
 std::cout << "sizeof(Wy__HStr)=" << sizeof(Wy__HStr) << std::endl;
 if((sizeof(Wy__HStr)%sizeof(double))!=0) {
   std::cout << "  Warning: sizeof(Wy__HStr) is not double aligned\n";
 }

 std::cout << std::dec
           << "  Wy__HStr::min_capacity()= " << Wy__HStr::min_capacity()
           << std::endl;
 std::cout << std::hex
           << "  Wy__HStr::max_capacity()= 0x" << Wy__HStr::max_capacity()
           << std::endl;

 if(std::string().max_size()!=Wy__HStr::max_capacity()) {
   std::cout << std::hex
           << "  std::string().max_size()= 0x" << std::string().max_size()
           << std::endl;
 }

 ck_str();
 ck_error();
 std::cout << "Checked OK\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr;
 print_ret(e);
 std::cerr << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr;
 std::cerr << chdr << "std::exception( \n" << e.what()
           << "\" )" << std::endl;
 std::cerr << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr << "unknow exception" << std::endl;
 throw;
};
