/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Build: make chk_rdque
*/

#include "wystr.h"
#include "wyregfile.h"
#include "wy_uty.h"
#include "wy__rdque.h"

static void ck_rd(WyByteFlow& bfile)
{
 WyRet r;
 {
   Wy__RdQue tmp(bfile);
   if(&tmp.rfile()!=&bfile) {
     WY_THROW( WyRet() );
   }
   if(tmp.qdata().size()!=0) {
     WY_THROW( WyRet() );
   }
   if((r=tmp.reset(bfile))!=OK) {
     WY_THROW(r);
   }
   if(tmp._rdsize()<512) {
     WY_THROW(r);
   }
   WyByteFlow bf_empty;
   if((r=tmp.reset(bf_empty))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if(&tmp.rfile()!=&bfile) {
     WY_THROW( WyRet() );
   }
 }

 // chk Wy__RdQue() error
 {
   WyByteFlow bf;
   try {
     Wy__RdQue tmp(bf);
     WY_THROW( WyRet() );
   }
   catch(const Wy__RdQue::Reply& e) {
     if(e!=Wym_EBADF) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   }
   catch(...) {
     throw;
   };
 }

 // chk getdata(WyCSeg&,size_t)
 {
   if(bfile.ftype()!=S_IFREG) {
     WY_THROW( WyRet() );       // this chk work for regular file only
   }
   // note: r/w position of bfile is assumed at 0
   //
   WyByteFlow tf(bfile);  // for contract.
   Wy__RdQue rdq(bfile);
   WyStr rbuf;
   WyCSeg rseg;

   for(;;) {
     size_t n_rd;
     if((r=rdq.getdata(rseg,5))!=OK) {
       WY_THROW(r);
     }
     rbuf.reset();
     if((r=tf.read(rbuf,5,n_rd))!=OK) {
       WY_THROW(r);
     }
     if(rbuf!=rseg) {
       WY_THROW( WyRet() );
     }
     if(n_rd==0) {
       break;
     }
   }
 }

 // chk getdata(char&)
 {
   if(bfile.ftype()!=S_IFREG) {
     WY_THROW( WyRet() );       // this chk work for regular file only
   }
   WyRegFile tf1(bfile);  // for contract
   tf1.set_pos(0);
   WyRegFile tf2(tf1);

   Wy__RdQue rdq(tf2);
   WyStr rbuf;
   char rch;

   for(;;) {
     size_t n_rd;

     rbuf.reset();
     if((r=tf1.read(rbuf,1,n_rd))!=OK) {
       WY_THROW(r);
     }
     if(n_rd!=1) {
       if(n_rd!=0) {
         WY_THROW( WyRet() );
       }
       if((r=rdq.getdata(rch))!=Wym_ENOENT) {
         WY_THROW(r);
       }
       break;
     }
     if((r=rdq.getdata(rch))!=OK) {
       WY_THROW(r);
     }
     if((rbuf.size()!=1)&&(rbuf[0]!=rch)) {
       WY_THROW( WyRet() );
     }
   }
 }

 // chk getdata(WyCSeg&,size_t,char)
 {
   if(bfile.ftype()!=S_IFREG) {
     WY_THROW( WyRet() );       // this chk work for regular file only
   }
   WyRegFile tf1(bfile);  // for contract
   tf1.set_pos(0);
   WyRegFile tf2(tf1);

   Wy__RdQue rdq(tf2);
   WyStr rbuf;
   WyCSeg rseg;

   for(;;) {
     size_t n_rd;

     rbuf.reset();
     if((r=tf1._read_till(rbuf,40,n_rd,'\n'))!=OK) {
       WY_THROW(r);
     }
     if((r=rdq.getdata(rseg,40,'\n'))!=OK) {
       WY_THROW(r);
     }
     if(rbuf!=rseg) {
       WY_THROW( WyRet() );
     }
     if(n_rd==0) {
       break;
     }
   }
 }

 // chk _feedque(size_t&)
 {
   if(bfile.ftype()!=S_IFREG) {
     WY_THROW( WyRet() );       // this chk work for regular file only
   }
   WyRegFile tf1(bfile);  // for contract
   tf1.set_pos(0);
   WyRegFile tf2(tf1);

   Wy__RdQue rdq(tf2);
   WyStr rbuf;
   WyCSeg rseg;
   rdq._rdsize(11);
   for(;;) {
     size_t n_rd,q_rd;

     if((r=tf1.read(rbuf,11,n_rd))!=OK) {
       WY_THROW(r);
     }
     if((r=rdq._feedque(q_rd))!=OK) {
       WY_THROW(r);
     }
     if(rbuf!=rdq.qdata()) {
       WY_THROW( WyRet() );
     }
     if(n_rd!=q_rd) {
       WY_THROW( WyRet() );
     }
     if(n_rd==0) {
       break;
     }
   }
 }

};

static void ck_rdq(void)
{
 WyRegFile tfile(__FILE__,O_RDONLY);
 ck_rd(tfile);
};

static const WyStr chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 Wy::cout << "Checking wy__rdque.h ...\n";
 ck_rdq();
 Wy::cout << "Checked OK\n";
 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << chdr << Wy::wrd(e) << "\n";
 return(-1);
}
catch(const std::exception& e) {
 Wy::cerr << chdr << "std::exception\n";
 return(-1);
}
catch(...) {
 Wy::cerr << chdr << "unknown unwind\n";
 return(-1);
};
