/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Build: make chk_reply
*/

#include "wyret.h"
#include <stdexcept>
#include <iostream>

#if WYR_REPLY_VERSION!=30
#error Check code is for WYR_REPLY_VERSION 30
#endif

static void t_reply(void)
{
#ifndef WY_DEBUG
 // Wy_RepInfo(const char*)
 {
  const char* Ustr="helo";
  Wy_ErrInfo tmp(Ustr);
  if(tmp.c_repcode()!=-1) {
    WY_THROW( WyRet() );
  }
  if(tmp.c_repstr()!=Ustr) {
    WY_THROW( WyRet() );
  }

  char buf[16]={100,};
  if(tmp.get_reply(NULL,1)!=4) {
    WY_THROW( WyRet() );
  }
  if(buf[0]!=100) {
    WY_THROW( WyRet() );
  }
  if(tmp.get_reply(buf,sizeof(buf))!=4) {
    WY_THROW( WyRet() );
  }
  if(::memcmp(Ustr,buf,4)!=0) {
    WY_THROW( WyRet() );
  }
  buf[0]=100;
  if(tmp.get_reply(buf,3)!=3) {
    WY_THROW( WyRet() );
  }
  if(::memcmp(Ustr,buf,3)!=0) {
    WY_THROW( WyRet() );
  }
 }

 // Wy_RepInfo(const char*,const char*)
 // set_ustr(..)
 {
  const char* Ustr="helo5";
  Wy_ErrInfo tmp(50,Ustr);
  if(tmp.c_repcode()!=50) {
    WY_THROW( WyRet() );
  }
  if(tmp.c_repstr()!=Ustr) {
    WY_THROW( WyRet() );
  }
  if(tmp.get_reply(NULL,0)!=5) {
    WY_THROW( WyRet() );
  }
  char buf[16]={100,};
  if(tmp.get_reply(buf,sizeof(buf))!=5) {
    WY_THROW( WyRet() );
  }
  if(memcmp(buf,Ustr,5)!=0) {
    WY_THROW( WyRet() );
  }
 }
#endif
};

static void chk_default(const WyReply& em)
{
 if(em.is_default()==false) {
   throw WyRet();
 }
 {
  WyReply tmp;
  if(em.operator->()!=tmp.operator->()) {
   throw WyRet();
  }
 }
 if(em->c_repcode()!=-1) {
   throw WyRet();
 }
 static const char* DeiMsg="General error";
 static const size_t DeiMsgLen=strlen(DeiMsg);
 if(strcmp(em->c_repstr(),DeiMsg)!=0) {
   throw WyRet();
 }

 char buf[16];
 if(em->get_reply(buf,sizeof(buf))!=DeiMsgLen) {
   throw WyRet();
 }
 if(memcmp(buf,DeiMsg,DeiMsgLen)!=0) {
   throw WyRet();
 }
};

#define CHK_DEFAULT(em) try { chk_default(em); } catch(const WyRet& e) {\
                            WY_THROW( WyRet(e) ); };

// Check the public interface of WyReply
//
static void t1(void)
{
 WyReply e1;            // default constructor
 WyReply e2(ENOENT);    // construct by errno
 WyReply e3(e2);        // constructor by copy
 WyReply e4(0);         // construct by errno (OK object)
 char buf[128];

 if(WyReply().operator->()!=e1.operator->()) {
   WY_THROW( WyRet() );
 }

 // Basic construct and compare
 //
 if(e1.is_default()==false) {
   WY_THROW(WyRet());
 }
 if(e2.is_default()==true) {
   WY_THROW( WyRet() );
 }
 if(e3.is_default()==true) {
   WY_THROW( WyRet() );
 }
 if(e1==e2) {
   WY_THROW( ::WyRet() );
 }
 if(e1!=e2) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e2!=e3) {
   WY_THROW( WyRet() );
 }
 if(e2==e3) {
 } else {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e1);

 // chk OK==WyReply(0)
 //
 if(WyReply(0)!=OK) {
   WY_THROW( WyRet() );
 }

 if(e1==OK) {
   WY_THROW( WyRet() );
 }
 if(e1!=OK) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e2==OK) {
   WY_THROW( WyRet() );
 }
 if(e2!=OK) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e4==OK) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e4!=OK) {
   WY_THROW( WyRet() );
 }
 if(e4.is_default()) {
   WY_THROW( WyRet() );
 }

 // chk WyReply(en)
 {
  WyReply tmp(Wy::_max_errno+1);
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  CHK_DEFAULT(tmp);

  WyReply tmp2(ENOENT);
  if(tmp2->c_repcode()!=ENOENT) {
    WY_THROW( WyRet() );
  };
 }

 // chk WyReply(Wy_RepInfo&) (manual example)
 {
  class AppErrInfo : public Wy_RepInfo {
    public:
      AppErrInfo(int ec, const char* estr) throw()
         : Wy_RepInfo(ec, estr) {};
      size_t get_reply(char* buf, size_t blen) const throw()
         { return Wy_RepInfo::get_reply(buf,blen); };
      void set_reply(const char*, size_t) throw()
         {};
  };

  static AppErrInfo e1(45,"Bad Input");

  const WyReply BadInput(e1);

  if(BadInput->c_repcode()!=45) {
    WY_THROW( WyRet() );
  }
  if(BadInput->get_reply(buf,sizeof(buf))!=9) {
    WY_THROW( WyRet() );
  }
  if(::memcmp(buf,e1.c_repstr(),9)!=0) {
    WY_THROW( WyRet() );
  }
 }

 // chk reset
 //
 e4.reset();
 CHK_DEFAULT(e4);

 if(e2.is_default()) {
   WY_THROW( WyRet() );
 }
 e4.reset(e2);
 if(e4!=e2) {
   WY_THROW( WyRet() );
 }
 e4.reset(Wym_EBADF);
 if(e4!=Wym_EBADF) {
   WY_THROW( WyRet() );
 }
 if(e4.reset(4)!=4) {
   WY_THROW( WyRet() );
 }
 if(e4->c_repcode()!=4) {
   WY_THROW( WyRet() );
 }
 if(e4.reset(Wy::_max_errno+1)!=-1) {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e4);
 e4.reset(Wym_EBADF);
 if(e4.reset(-2)!=-1) {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e4);

 // chk operator=
 //
 if(e4==e2) {
   WY_THROW( WyRet() );
 }
 e4=e2;
 if(e4!=e2) {
   WY_THROW( WyRet() );
 }
 e4=Wym_EBADF;
 if(e4!=Wym_EBADF) {
   WY_THROW( WyRet() );
 }

 if(e4.is_default()) {
   WY_THROW( WyRet() );
 }
 e4=200;
 CHK_DEFAULT(e4);

 // chk some c_repcode
 //
 if(Wym_EPERM->c_repcode()!=EPERM) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOENT->c_repcode()!=ENOENT) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESRCH->c_repcode()!=ESRCH) {
   WY_THROW( WyRet() );
 }
 if(Wym_EINTR->c_repcode()!=EINTR) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENETRESET->c_repcode()!=ENETRESET) {
   WY_THROW( WyRet() );
 }
 if(Wym_EMEDIUMTYPE->c_repcode()!=EMEDIUMTYPE) {
   WY_THROW( WyRet() );
 }
 if(Wym_E127->c_repcode()!=127) {
   WY_THROW( WyRet() );
 }

 // chk set_reply(..)/get_reply(..)
 //
 {
   char buf[64];
   if(OK->get_reply(buf,sizeof(buf))!=7) {
     WY_THROW( WyRet() );
   }
   if(memcmp(buf,"Success",7)!=0) {
     WY_THROW( WyRet() );
   }
   const char Ustr[]="Test Msg for OK";
   OK->set_reply(Ustr,sizeof(Ustr));
   if(OK->get_reply(buf,sizeof(buf))!=sizeof(Ustr)) {
     WY_THROW( WyRet() );
   }
   if(memcmp(buf,Ustr,sizeof(Ustr))!=0) {
     WY_THROW( WyRet() );
   }
   OK->set_reply("Success",8);  // including trailing zero

   if(OK->c_repcode()!=0) {
     WY_THROW( WyRet() );
   }
 }

 // chk move ctor
 {
  WyReply raw_buf[2];      // ok WyReply is discardable
  ::memset(raw_buf,0,sizeof(raw_buf));
  WyReply* const p1( reinterpret_cast<WyReply*>(&raw_buf[0]) );
  WyReply* const p2( reinterpret_cast<WyReply*>(&raw_buf[1]) );

  new(p1) WyReply(Wym_ENOENT);
  if(*p1!=Wym_ENOENT) {
    p1->~WyReply();
    WY_THROW( WyRet() );
  }

  // move *p1 to p2
  new(p2) WyReply(*p1,Wy::ByMove);
  if(*p2!=Wym_ENOENT) {
    p2->~WyReply();
    WY_THROW( WyRet() );
  }
  p2->~WyReply();
 }
};

// Check the mapped errno 0-127
//
static void t2(void)
{
 if((OK!=WyReply(0))||(OK->c_repcode()!=0)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EPERM!=WyReply(EPERM))||(Wym_EPERM->c_repcode()!=EPERM)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOENT!=WyReply(ENOENT))||(Wym_ENOENT->c_repcode()!=ENOENT)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ESRCH!=WyReply(ESRCH))||(Wym_ESRCH->c_repcode()!=ESRCH)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EINTR!=WyReply(EINTR))||(Wym_EINTR->c_repcode()!=EINTR)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EIO!=WyReply(EIO))||(Wym_EIO->c_repcode()!=EIO)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENXIO!=WyReply(ENXIO))||(Wym_ENXIO->c_repcode()!=ENXIO)) {
   WY_THROW( WyRet() );
 }
 if((Wym_E2BIG!=WyReply(E2BIG))||(Wym_E2BIG->c_repcode()!=E2BIG)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOEXEC!=WyReply(ENOEXEC))||(Wym_ENOEXEC->c_repcode()!=ENOEXEC)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EBADF!=WyReply(EBADF))||(Wym_EBADF->c_repcode()!=EBADF)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ECHILD!=WyReply(ECHILD))||(Wym_ECHILD->c_repcode()!=ECHILD)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EAGAIN!=WyReply(EAGAIN))||(Wym_EAGAIN->c_repcode()!=EAGAIN)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOMEM!=WyReply(ENOMEM))||(Wym_ENOMEM->c_repcode()!=ENOMEM)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EACCES!=WyReply(EACCES))||(Wym_EACCES->c_repcode()!=EACCES)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EFAULT!=WyReply(EFAULT))||(Wym_EFAULT->c_repcode()!=EFAULT)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOTBLK!=WyReply(ENOTBLK))||(Wym_ENOTBLK->c_repcode()!=ENOTBLK)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EBUSY!=WyReply(EBUSY))||(Wym_EBUSY->c_repcode()!=EBUSY)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EEXIST!=WyReply(EEXIST))||(Wym_EEXIST->c_repcode()!=EEXIST)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EXDEV!=WyReply(EXDEV))||(Wym_EXDEV->c_repcode()!=EXDEV)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENODEV!=WyReply(ENODEV))||(Wym_ENODEV->c_repcode()!=ENODEV)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOTDIR!=WyReply(ENOTDIR))||(Wym_ENOTDIR->c_repcode()!=ENOTDIR)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EISDIR!=WyReply(EISDIR))||(Wym_EISDIR->c_repcode()!=EISDIR)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EINVAL!=WyReply(EINVAL))||(Wym_EINVAL->c_repcode()!=EINVAL)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENFILE!=WyReply(ENFILE))||(Wym_ENFILE->c_repcode()!=ENFILE)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EMFILE!=WyReply(EMFILE))||(Wym_EMFILE->c_repcode()!=EMFILE)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOTTY!=WyReply(ENOTTY))||(Wym_ENOTTY->c_repcode()!=ENOTTY)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ETXTBSY!=WyReply(ETXTBSY))||(Wym_ETXTBSY->c_repcode()!=ETXTBSY)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EFBIG!=WyReply(EFBIG))||(Wym_EFBIG->c_repcode()!=EFBIG)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOSPC!=WyReply(ENOSPC))||(Wym_ENOSPC->c_repcode()!=ENOSPC)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ESPIPE!=WyReply(ESPIPE))||(Wym_ESPIPE->c_repcode()!=ESPIPE)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EROFS!=WyReply(EROFS))||(Wym_EROFS->c_repcode()!=EROFS)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EMLINK!=WyReply(EMLINK))||(Wym_EMLINK->c_repcode()!=EMLINK)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EPIPE!=WyReply(EPIPE))||(Wym_EPIPE->c_repcode()!=EPIPE)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EDOM!=WyReply(EDOM))||(Wym_EDOM->c_repcode()!=EDOM)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ERANGE!=WyReply(ERANGE))||(Wym_ERANGE->c_repcode()!=ERANGE)) {
   WY_THROW( WyRet() );
 }
 if((Wym_EDEADLK!=WyReply(EDEADLK))||(Wym_EDEADLK->c_repcode()!=EDEADLK)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENAMETOOLONG!=WyReply(ENAMETOOLONG))||(Wym_ENAMETOOLONG->c_repcode()!=ENAMETOOLONG)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOLCK!=WyReply(ENOLCK))||(Wym_ENOLCK->c_repcode()!=ENOLCK)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOSYS!=WyReply(ENOSYS))||(Wym_ENOSYS->c_repcode()!=ENOSYS)) {
   WY_THROW( WyRet() );
 }
 if((Wym_ENOTEMPTY!=WyReply(ENOTEMPTY))||(Wym_ENOTEMPTY->c_repcode()!=ENOTEMPTY)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELOOP!=WyReply(ELOOP)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EWOULDBLOCK!=WyReply(EWOULDBLOCK)) {
   WY_THROW( WyRet() );
 }
 if(Wym_E41!=WyReply(41)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOMSG!=WyReply(ENOMSG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EIDRM!=WyReply(EIDRM)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ECHRNG!=WyReply(ECHRNG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EL2NSYNC!=WyReply(EL2NSYNC)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EL3HLT!=WyReply(EL3HLT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EL3RST!=WyReply(EL3RST)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELNRNG!=WyReply(ELNRNG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EUNATCH!=WyReply(EUNATCH)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOCSI!=WyReply(ENOCSI)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EL2HLT!=WyReply(EL2HLT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADE!=WyReply(EBADE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADR!=WyReply(EBADR)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EXFULL!=WyReply(EXFULL)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOANO!=WyReply(ENOANO)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADRQC!=WyReply(EBADRQC)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADSLT!=WyReply(EBADSLT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EDEADLOCK!=WyReply(EDEADLOCK)) {
   WY_THROW( WyRet() );
 }
 if(Wym_E58!=WyReply(58)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBFONT!=WyReply(EBFONT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOSTR!=WyReply(ENOSTR)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENODATA!=WyReply(ENODATA)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ETIME!=WyReply(ETIME)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOSR!=WyReply(ENOSR)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENONET!=WyReply(ENONET)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOPKG!=WyReply(ENOPKG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EREMOTE!=WyReply(EREMOTE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOLINK!=WyReply(ENOLINK)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EADV!=WyReply(EADV)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESRMNT!=WyReply(ESRMNT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ECOMM!=WyReply(ECOMM)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EPROTO!=WyReply(EPROTO)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EMULTIHOP!=WyReply(EMULTIHOP)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EDOTDOT!=WyReply(EDOTDOT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADMSG!=WyReply(EBADMSG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EOVERFLOW!=WyReply(EOVERFLOW)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOTUNIQ!=WyReply(ENOTUNIQ)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EBADFD!=WyReply(EBADFD)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EREMCHG!=WyReply(EREMCHG)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELIBACC!=WyReply(ELIBACC)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELIBBAD!=WyReply(ELIBBAD)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELIBSCN!=WyReply(ELIBSCN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELIBMAX!=WyReply(ELIBMAX)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ELIBEXEC!=WyReply(ELIBEXEC)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EILSEQ!=WyReply(EILSEQ)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ERESTART!=WyReply(ERESTART)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESTRPIPE!=WyReply(ESTRPIPE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EUSERS!=WyReply(EUSERS)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOTSOCK!=WyReply(ENOTSOCK)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EDESTADDRREQ!=WyReply(EDESTADDRREQ)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EMSGSIZE!=WyReply(EMSGSIZE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EPROTOTYPE!=WyReply(EPROTOTYPE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOPROTOOPT!=WyReply(ENOPROTOOPT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EPROTONOSUPPORT!=WyReply(EPROTONOSUPPORT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESOCKTNOSUPPORT!=WyReply(ESOCKTNOSUPPORT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EOPNOTSUPP!=WyReply(EOPNOTSUPP)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EPFNOSUPPORT!=WyReply(EPFNOSUPPORT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EAFNOSUPPORT!=WyReply(EAFNOSUPPORT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EADDRINUSE!=WyReply(EADDRINUSE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EADDRNOTAVAIL!=WyReply(EADDRNOTAVAIL)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENETDOWN!=WyReply(ENETDOWN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENETUNREACH!=WyReply(ENETUNREACH)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENETRESET!=WyReply(ENETRESET)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ECONNABORTED!=WyReply(ECONNABORTED)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ECONNRESET!=WyReply(ECONNRESET)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOBUFS!=WyReply(ENOBUFS)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EISCONN!=WyReply(EISCONN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOTCONN!=WyReply(ENOTCONN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESHUTDOWN!=WyReply(ESHUTDOWN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ETOOMANYREFS!=WyReply(ETOOMANYREFS)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ETIMEDOUT!=WyReply(ETIMEDOUT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ECONNREFUSED!=WyReply(ECONNREFUSED)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EHOSTDOWN!=WyReply(EHOSTDOWN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EHOSTUNREACH!=WyReply(EHOSTUNREACH)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EALREADY!=WyReply(EALREADY)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EINPROGRESS!=WyReply(EINPROGRESS)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESTALE!=WyReply(ESTALE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EUCLEAN!=WyReply(EUCLEAN)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOTNAM!=WyReply(ENOTNAM)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENAVAIL!=WyReply(ENAVAIL)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EISNAM!=WyReply(EISNAM)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EREMOTEIO!=WyReply(EREMOTEIO)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EDQUOT!=WyReply(EDQUOT)) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOMEDIUM!=WyReply(ENOMEDIUM)) {
   WY_THROW( WyRet() );
 }
 if(Wym_EMEDIUMTYPE!=WyReply(EMEDIUMTYPE)) {
   WY_THROW( WyRet() );
 }
 if(Wym_E125!=WyReply(125)) {
   WY_THROW( WyRet() );
 }
 if(Wym_E126!=WyReply(126)) {
   WY_THROW( WyRet() );
 }
 if(Wym_E127!=WyReply(127)) {
   WY_THROW( WyRet() );
 }

 // chk EWOULDBLOCK EAGAIN
 if(Wym_EWOULDBLOCK!=Wym_EAGAIN) {  // EWOULDBLOCK and EAGAIN should be identical
   WY_THROW( WyRet() );
 }
 if(Wym_EDEADLK!=Wym_EDEADLOCK) {   // EDEADLOCK and EDEADLK should be identical
   WY_THROW( WyRet() );
 }
 if(Wym_EWOULDBLOCK->c_repcode()!=EAGAIN) {
   WY_THROW( WyRet() );
 }
 if(Wym_EDEADLK->c_repcode()!=EDEADLK) {
   WY_THROW( WyRet() );
 }

 // chkeck c_repstr
 //
 std::cout << "Different string from ::strerror(errno):\n";
 //setlocale(LC_ALL,"");
 for(int i=0; i<=Wy::_max_errno; ++i) {
   WyReply ei(i);
   if(std::strcmp(::strerror(i),ei->c_repstr())!=0) {
     std::cout << "\n  strerror(" << i << ")=" << ::strerror(i)
               << "\n  c_repstr()=" << ei->c_repstr() << std::endl;
   }
 }
}

static void t_error(void)
{
 WyReply tmp(-3);
 if(tmp.is_default()==false) {
   WY_THROW(WyRet());
 }
 tmp.reset(-3);    // supress unused warning
 if(tmp.is_default()==false) {
   WY_THROW(WyRet());
 }

 WyReply tmp1;
 if(tmp1.is_default()==false) {
   WY_THROW(WyRet());
 }
 tmp1=-4;
 if(tmp1.is_default()==false) {
   WY_THROW(WyRet());
 }
};

static void t_locale_reply(void)
{
#ifdef LC_ALL
 // dump locale specific error string
 const char* loc=setlocale(LC_ALL,"");
 if(loc==NULL) {
   WY_THROW( WyRet() );
 }
 std::cout << "\nLocale used reply string:\n";
 
 const char* cstr;
 cstr=::strerror(OK->c_repcode());
 OK->set_reply(cstr,::strlen(cstr)+1);  // including trailing zero
 cstr=::strerror(Wym_EPERM->c_repcode());
 Wym_EPERM->set_reply(cstr,::strlen(cstr)+1);
 cstr=::strerror(Wym_ENOENT->c_repcode());
 Wym_ENOENT->set_reply(cstr,::strlen(cstr)+1);
 cstr=::strerror(Wym_ESRCH->c_repcode());
 Wym_ESRCH->set_reply(cstr,::strlen(cstr)+1);
 cstr=::strerror(Wym_EBADMSG->c_repcode());
 Wym_EBADMSG->set_reply(cstr,::strlen(cstr)+1);
 cstr=::strerror(Wym_E127->c_repcode());
 Wym_E127->set_reply(cstr,::strlen(cstr)+1);

 char buf[128];
 memset(buf,sizeof(buf),0);

 buf[ OK->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << "          OK => " << buf << std::endl;

 buf[ Wym_EPERM->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << "   Wym_EPERM => " << buf << std::endl;

 buf[ Wym_ENOENT->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << "  Wym_ENOENT => " << buf << std::endl;

 buf[ Wym_ESRCH->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << "   Wym_ESRCH => " << buf << std::endl;

 buf[ Wym_EBADMSG->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << " Wym_EBADMSG => " << buf << std::endl;

 buf[ Wym_E127->get_reply(buf,sizeof(buf)-1) ]=0;
 std::cout << "    Wym_E127 => " << buf << std::endl;

 std::cout << "    ........." << std::endl;
 setlocale(LC_ALL,loc);
#endif
};

static const std::string chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WyReply ...\n";
 t_reply();
 t1(); 
 t2();
 t_error();
 t_locale_reply();

 std::cout << "Checked OK\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr << "WyRet thrown" << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr << "unknown unwind" << std::endl;
 return(-1);
}
