/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/
#include "wytimespec.h"
#include "wystr.h"
#include <iostream>
#include <limits>
#include <unistd.h>
#include <sys/types.h>

#if WYTIMESPEC_VERSION!=30
#error Test code is for WYTIMESPEC_VERSION 30
#endif

#define TEST_SLEEP_TILL

//
// Constant limits use for test
//
static const long LongMax=std::numeric_limits<long>::max();
static const long LongMin=std::numeric_limits<long>::min();
static const Wy_Second MaxSecond=std::numeric_limits<Wy_Second>::max();
static const Wy_Second MinSecond=std::numeric_limits<Wy_Second>::min();
static const Wy_Second MaxL2=MaxSecond/2;
static const Wy_Second MaxNano=std::numeric_limits<Wy_Nano>::max();
static const Wy_Second MinNano=std::numeric_limits<Wy_Nano>::min();

static bool chk_default(const WyTimeSpec& tm)
{
const static WyTimeSpec T0;
 if(tm.is_default()==false) {
   return(false);
 }
 if((tm.second()!=0)||(tm.nano()!=0)) {
   return(false);
 }
 if(tm==T0) {
 } else {
   return(false);
 }
 if(tm!=T0) {
   return(false);
 }
 return(true);
};

static bool chk_not_default(const WyTimeSpec& tm)
{
const static WyTimeSpec T0;
 if(tm.is_default()) {
   return(false);
 }
 if((tm.second()==0)&&(tm.nano()==0)) {
   return(false);
 }
 if(tm!=T0) {
 } else {
   return(false);
 }
 if(tm==T0) {
   return(false);
 }
 return(true);
};

static bool chk_equ(const WyTimeSpec& tm1,const WyTimeSpec& tm2)
{
 if(tm1==tm2) {
   if(tm1.second()!=tm2.second()) {
     WY_THROW( WyRet() );
   }
   if(tm1.nano()!=tm2.nano()) {
     WY_THROW( WyRet() );
   }
 } else {
   return(false);
 }
 if(tm1!=tm2) {
   return(false);
 }
 if(tm1.second()!=tm2.second()) {
   return(false);
 }
 if(tm1.nano()!=tm2.nano()) {
   return(false);
 }
 return(true);
};

#define CHK_EQU(expr1,expr2) if(chk_equ(expr1,expr2)==false) { WY_THROW( WyRet() ); };
#define CHK_NOT_EQU(expr1,expr2) if(chk_not_equ(expr1,expr2)==false) { WY_THROW( WyRet() ); };

#define CHK_DEFAULT(expr) if(chk_default(expr)==false) { WY_THROW( WyRet() ); };
#define CHK_NOT_DEFAULT(expr) if(chk_not_default(expr)==false) { WY_THROW( WyRet() ); };

static void dump_tm(const struct tm* tsp)
{
 std::cout 
     << "tm_year=" << tsp->tm_year
     << ", tm_mon=" << tsp->tm_mon
     << ", tm_mday=" << tsp->tm_mday
     << ", tm_hour=" << tsp->tm_hour
     << ", tm_min=" << tsp->tm_min
     << ", tm_sec=" << tsp->tm_sec
     << ", tm_wday=" << tsp->tm_wday
     << ", tm_yday=" << tsp->tm_yday
     << ", tm_isdst=" << tsp->tm_isdst
     << "\n";
};

/*
   Check WyTimeSpec public interface (set_systime is not tested)
*/
static void t1(void)
{
 WyRet r;

 // Check WyTimeSpec()/WyTimeSpec(const WyTimeSpec&)
 //
 {
   WyTimeSpec a0,a1(a0);
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);
   CHK_EQU(a0,a1);
 }

 // WyTimeSpec(Wy_Second,Wy_Nano)/WyTimeSpec(Wy_Second)
 {
   WyTimeSpec a1(100,100);
   WyTimeSpec a2(a1);
   WyTimeSpec a3(99,1000000100L);
   CHK_NOT_DEFAULT(a1);
   CHK_NOT_DEFAULT(a2);
   CHK_NOT_DEFAULT(a3);
   CHK_EQU(a1,a2);
   CHK_EQU(a1,a3);
   if((a1.second()!=100)||(a1.nano()!=100)) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec a4(-100,-100);
   if((a4.second()!=-100)||(a4.nano()!=-100)) {
     WY_THROW( WyRet() );
   }
   WyTimeSpec a5(-100, 100);
   if((a5.second()!=-99)||(a5.nano()!=-WyTimeSpec::_Giga+100)) {
     std::cout << " second()=" << a5.second() 
               << " nano()=" << a5.nano() << std::endl;
     WY_THROW( WyRet() );
   }
   WyTimeSpec a6( 100,-100);
   if((a6.second()!=99)||(a6.nano()!=WyTimeSpec::_Giga-100)) {
     std::cout << " second()=" << a6.second() 
               << " nano()=" << a6.nano() << std::endl;
     WY_THROW( WyRet() );
   }

   WyTimeSpec a7(100,0);
   if((a7.second()!=100)||(a7.nano()!=0)) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec a8(-100,0);
   if((a8.second()!=-100)||(a8.nano()!=0)) {
     WY_THROW( WyRet() );
   }
 }

 // WyTimeSpec(const char*)
 {
   WyTimeSpec t1("234");
   CHK_NOT_DEFAULT(t1);
   if(t1.second()!=234) {
     WY_THROW( WyRet() );
   }
   if(t1.nano()!=0) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec t2("-2345.");
   CHK_NOT_DEFAULT(t2);
   if(t2.second()!=-2345) {
     WY_THROW( WyRet() );
   }
   if(t2.nano()!=0) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec t3("-.123456789");
   CHK_NOT_DEFAULT(t3);
   if(t3.second()!=0) {
     WY_THROW( WyRet() );
   }
   if(t3.nano()!=-123456789L) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec t4("12.34000000000000000");
   CHK_NOT_DEFAULT(t4);
   if(t4.second()!=12) {
     WY_THROW( WyRet() );
   }
   if(t4.nano()!=340000000L) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec t5("-2.345e1");
   CHK_NOT_DEFAULT(t5);
   if(t5.second()!=-23) {
     WY_THROW( WyRet() );
   }
   if(t5.nano()!=-450000000) {
     WY_THROW( WyRet() );
   }
 }

 // chk WyTimeSpec(const WyCSeg&)
 {
   WyTimeSpec t5(WyCSeg("-2.345e1x",8));
   CHK_NOT_DEFAULT(t5);
   if(t5.second()!=-23) {
     WY_THROW( WyRet() );
   }
   if(t5.nano()!=-450000000) {
     WY_THROW( WyRet() );
   }
 }

 // chk WyTimeSpec(double)
 {
   WyTimeSpec tmp( double(123.456) );
   CHK_NOT_DEFAULT(tmp);
   if(tmp.second()!=123) {
     WY_THROW( WyRet() );
   }
   if(tmp.nano()!=456000000) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec tmp2( double(-123.456) );
   CHK_NOT_DEFAULT(tmp2);
   if(tmp2.second()!=-123) {
     WY_THROW( WyRet() );
   }
   if(tmp2.nano()!=-456000000) {
     WY_THROW( WyRet() );
   }

   WyTimeSpec tmp3( double(0.456) );
   CHK_NOT_DEFAULT(tmp3);
   if(tmp3.second()!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp3.nano()!=456000000) {
     WY_THROW( WyRet() );
   }
 }

 // reset
 //
 {
   const WyTimeSpec a0;
   const WyTimeSpec a1(200,200);
   WyTimeSpec tmp(a1);
   CHK_DEFAULT(a0);
   CHK_NOT_DEFAULT(a1);
   CHK_NOT_DEFAULT(tmp);
   
   tmp.reset();
   CHK_DEFAULT(tmp);
   tmp.reset(a1);
   CHK_EQU(tmp,a1);
   tmp.reset(a0);
   CHK_EQU(tmp,a0);
   if((r=tmp.reset(a1.second(),a1.nano()))!=OK) {
      WY_THROW(r);
   }
   CHK_EQU(tmp,a1);

   tmp.reset(Wy_Second(),0);
   if(tmp.second()!=0) {
     WY_THROW( WyRet() );
   }

   if((r=tmp.reset(300,LongMax))!=OK) {
     WY_THROW( WyRet() ); 
   }
   if((tmp.second()!=300+LongMax/WyTimeSpec::_Giga)||
        (tmp.nano()!=LongMax%WyTimeSpec::_Giga)) {
    WY_THROW( WyRet() );   // not rounded-up
   }

   if((r=tmp.reset(300,LongMin))!=OK) {
     WY_THROW( WyRet() ); 
   }
   WyTimeSpec tmp1(300+LongMin/WyTimeSpec::_Giga,LongMin%WyTimeSpec::_Giga);
   CHK_EQU(tmp,tmp1);

   tmp.reset(20,0);
   if((tmp.second()!=20)||(tmp.nano()!=0)) {
     WY_THROW( WyRet() ); 
   }
 }

 // reset(const char*)
 {
   const WyTimeSpec NDefault(1,1);
   WyTimeSpec t1;
   CHK_DEFAULT(t1);

   t1.reset(NDefault);
   if((r=t1.reset("0"))!=OK) {
     WY_THROW(r);
   }
   CHK_DEFAULT(t1);

   t1.reset(NDefault);
   if((r=t1.reset("0."))!=OK) {
     WY_THROW(r);
   }
   CHK_DEFAULT(t1);

   t1.reset(NDefault);
   if((r=t1.reset(".0"))!=OK) {
     WY_THROW(r);
   }
   CHK_DEFAULT(t1);

   t1.reset(NDefault);
   if((r=t1.reset("0.0"))!=OK) {
     WY_THROW(r);
   }
   CHK_DEFAULT(t1);

   t1.reset(NDefault);
   if((r=t1.reset("-0.0"))!=OK) {
     WY_THROW(r);
   }
   CHK_DEFAULT(t1);

   if((r=t1.reset("1.000000002"))!=OK) {
     WY_THROW(r);
   }
   if(t1.second()!=1) {
     WY_THROW( WyRet() );
   }
   if(t1.nano()!=2) {
     WY_THROW( WyRet() );
   }

   if((r=t1.reset("-123.400000000000000000"))!=OK) {
     WY_THROW(r);
   }
   if(t1.second()!=-123) {
     WY_THROW( WyRet() );
   }
   if(t1.nano()!=-400000000L) {
     WY_THROW( WyRet() );
   }
 }

 // swap
 {
  const WyTimeSpec Nobj(1,2);
  WyTimeSpec tmp0(Nobj),tmp1,tmp2;
  CHK_NOT_DEFAULT(tmp0);
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);
  CHK_NOT_DEFAULT(Nobj);

  tmp1.swap(tmp2);
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);

  tmp1.swap(tmp0);
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_EQU(tmp1,Nobj);

  tmp0.swap(tmp1);
  CHK_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp0);
  CHK_EQU(tmp0,Nobj);
 }

 // operator =
 //
 {
   WyTimeSpec a2(-100,-100),a3(100,100),a4;
   CHK_DEFAULT(a4);
   a4=a3;
   CHK_EQU(a4,a3);
   a4=a2;
   CHK_EQU(a4,a2);
 }

 // operator =(const char*)
 {
   WyTimeSpec t1;
   CHK_DEFAULT(t1);
   if(&(t1="0")!=&t1) {
     WY_THROW( WyRet() );
   }
   t1="2.5e0";
   if(t1.second()!=2) {
     WY_THROW( WyRet() );
   }
   if(t1.nano()!=500000000L) {
     WY_THROW( WyRet() );
   }
   t1="-2.5";
   if(t1.second()!=-2) {
     WY_THROW( WyRet() );
   }
   if(t1.nano()!=-500000000L) {
     WY_THROW( WyRet() );
   }
 }

 // operator -()
 //
 {
   WyTimeSpec a1,a2;
   CHK_DEFAULT(a1);
   CHK_EQU(-a1,WyTimeSpec(Wy_Second(),Wy_Nano()));

   a1.reset(-111,-111);
   a2.reset(a1);
   CHK_EQU(-a2,WyTimeSpec(111,111));
   CHK_EQU(a1,a2);

   a1.reset(-111,-LongMax);
   a2.reset(a1);
   CHK_EQU(-a2,WyTimeSpec(111,LongMax));
   CHK_EQU(a1,a2);
 }

 // operator ++() --()
 {
   WyTimeSpec a1;
   CHK_DEFAULT(a1);
   if(++a1!=WyTimeSpec(1,Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
   if(++a1!=WyTimeSpec(2,Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
   if(--a1!=WyTimeSpec(1,Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
   if(--a1!=WyTimeSpec(Wy_Second(),Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
 }

 // operator ++(int) --(int)
 {
   WyTimeSpec a1;
   CHK_DEFAULT(a1);
   if(a1++!=WyTimeSpec()) {
     WY_THROW( WyRet() ); 
   }
   if(a1++!=WyTimeSpec(1,Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
   if(a1--!=WyTimeSpec(2,Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
   if(a1--!=WyTimeSpec(Wy_Second(1),Wy_Nano())) {
     WY_THROW( WyRet() ); 
   }
 }

 // operator +/-, +=
 //
 {
   WyTimeSpec a2,a3,a4;
   a2.reset(100,100);
   a3.reset(200,200);
   a4=a2+a3;
   if((a4.second()!=300)||(a4.nano()!=300)) {
     WY_THROW( WyRet() ); 
   }
   a4=a2-a3;
   CHK_EQU(a4,WyTimeSpec(-100,-100));

   a4+=a3;
   CHK_EQU(a4,a2);
 }

 // operator <,<=, >,>=
 //
 {
   WyTimeSpec a2,a3,a4;
   a2.reset(100,100);
   a3.reset(200,200);
   a4=a2+a3;
   if((a4<a2)||(a4<a3)) {
     WY_THROW( WyRet() ); 
   }
   if((a4<=a2)||(a4<=a3)) {
     WY_THROW( WyRet() ); 
   }
   a2.reset(-100,-100);
   a3.reset(-200,-200);
   a4=a2+a3;
   if((a4>a2)||(a4>a3)) {
     WY_THROW( WyRet() ); 
   }
   if((a4>=a2)||(a4>=a3)) {
     WY_THROW( WyRet() ); 
   }
 }

 // abs
 {
   WyTimeSpec a2(1,2),a3(-3,-4);
   if(a2.abs()!=a2) {
     WY_THROW( WyRet() ); 
   }
   if(a3.abs()!=WyTimeSpec(3,4)) {
     WY_THROW( WyRet() ); 
   }
   if(a2!=WyTimeSpec(1,2)) {
     WY_THROW( WyRet() ); 
   }
   if(a3!=WyTimeSpec(-3,-4)) {
     WY_THROW( WyRet() ); 
   }
 }

 // add
 //
 {
   WyTimeSpec a2,a3;
   if((r=a2.reset(-400,-400))!=OK) {
     WY_THROW(r); 
   }
   a3.reset(a2);
   if((r=a2.add(500,500))!=OK) {
     WY_THROW(r); 
   }
   if((r=a3.add(WyTimeSpec(500,500)))!=OK) {
     WY_THROW(r); 
   }
   CHK_EQU(a2,a3);

   if((a2.nano()!=100)||(a2.second()!=100)) {
     WY_THROW( WyRet() ); 
   }
   if((r=a2.add(a2))!=OK) {
     WY_THROW(r); 
   }
   if((a2.nano()!=200)||(a2.second()!=200)) {
     WY_THROW( WyRet() ); 
   }
 }

 // sub
 //
 {
   WyTimeSpec a2,a3;
   if((r=a2.reset(-400,-400))!=OK) {
     WY_THROW(r); 
   }
   a3.reset(a2);
   if((r=a2.sub(500,500))!=OK) {
     WY_THROW(r); 
   }
   if((r=a3.sub(WyTimeSpec(500,500)))!=OK) {
     WY_THROW(r); 
   }
   CHK_EQU(a2,a3);

   if((a2.nano()!=-900)||(a2.second()!=-900)) {
     WY_THROW( WyRet() ); 
   }
   if((r=a2.sub(a2))!=OK) {
     WY_THROW(r); 
   }
   CHK_DEFAULT(a2);

   a2.reset( WyTimeSpec(MinSecond,0) );
   a3=a2;
   if((r=a2.sub(WyTimeSpec(0,0)))!=OK) {
     WY_THROW(r); 
   }
   if(a2!=a3) {
     WY_THROW( WyRet() ); 
   }
 }

 // mul
 {
  const signed long SS(100000), NN(999999999);
  WyTimeSpec tmp(SS,NN);
  if((r=tmp.mul((signed long)10000))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=WyTimeSpec(1000009999,999990000)) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.div((signed long)10000))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=WyTimeSpec(SS,NN)) {
    WY_THROW( WyRet() ); 
  }

  tmp.reset(-SS,-NN);
  if((r=tmp.mul((signed long)10000))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=WyTimeSpec(-1000009999,-999990000)) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.div((signed long)10000))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=WyTimeSpec(-SS,-NN)) {
    WY_THROW( WyRet() ); 
  }

  tmp.reset(-SS,-NN);
  if((r=tmp.mul((signed long)1000000000))!=Wym_ERANGE) {
    WY_THROW(r); 
  }
  if((r=tmp.div((signed long)0))!=Wym_EDIVZERO) {
    WY_THROW(r); 
  }

  const WyTimeSpec CT(MinSecond,0);
  tmp.reset(CT);
  if((r=tmp.mul((long)1))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=CT) {
    WY_THROW( WyRet() ); 
  }
 }

 // mul(const WyTimeSpec&), div(const WyTimeSpec&)
 {
  const WyTimeSpec t1(123,456000000L);
  WyTimeSpec tmp(t1);
  if((r=tmp.mul(t1))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=WyTimeSpec(15241,383936000L)) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.mul( WyTimeSpec(0,-500000000L)))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=WyTimeSpec(- 7620, -691968000L)) {
    WY_THROW( WyRet() ); 
  }

  tmp.reset(std::numeric_limits<WyTimeSpec>::max());
  if((r=tmp.mul( WyTimeSpec(1,0) ))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=std::numeric_limits<WyTimeSpec>::max()) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.mul( WyTimeSpec(-1,0) ))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(-tmp!=std::numeric_limits<WyTimeSpec>::max()) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.mul( WyTimeSpec(-1,0) ))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=std::numeric_limits<WyTimeSpec>::max()) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.mul( WyTimeSpec(0,999999999) ))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp>=std::numeric_limits<WyTimeSpec>::max()) {
    WY_THROW( WyRet() ); 
  }

/*
  if((r=tmp.div( WyTimeSpec(0,-500000000L)))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=WyTimeSpec(15241,383936000L)) {
    std::cerr << tmp.second() << ",  " << tmp.nano() << '\n';
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.div(t1))!=OK) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=t1) {
    WY_THROW( WyRet() ); 
  }
*/
  tmp=t1;
  if((r=tmp.mul( WyTimeSpec(MaxSecond,0)))!=Wym_ERANGE) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=t1) {
    WY_THROW( WyRet() ); 
  }
  if((r=tmp.mul( WyTimeSpec(MinSecond,0)))!=Wym_ERANGE) {
    WY_THROW( WyRet() ); 
  }
  if(tmp!=t1) {
    WY_THROW( WyRet() ); 
  }

 }

 // mul/div
 {
  WyTimeSpec tmp(1,MinNano);
  if(tmp.mul((long)999)!=OK) {
    WY_THROW(r);
  }
  if(tmp.div((long)999)!=OK) {
    WY_THROW(r);
  }
  if(tmp!=WyTimeSpec(1,MinNano)) {
     WY_THROW( WyRet() );
  }

  const WyTimeSpec CT(MinSecond,0);
  tmp.reset(CT);
  if((r=tmp.div(1))!=OK) {
    WY_THROW(r); 
  }
  if(tmp!=CT) {
    WY_THROW( WyRet() ); 
  }
 }

 // Simple test for of Wy::now
 //
 {
  //std::cout << Wy::wrd(Wy::now()) << std::endl;
  WyTimeSpec T0(1100966810,Wy_Nano());   // time this code written
  WyTimeSpec T_3year(T0+WyTimeSpec(3*365L*24*3600,0)); // 3 years life time
  WyTimeSpec tmp=Wy::now();

  if(tmp<=T0) {
    std::cerr << "\a\aSystem time is long time ago, you might be not care\n";
    r=OK;
    WY_HERE(r); std::cerr << Wy::wrd(r).c_str() << std::endl;
  }
  if(tmp>=T_3year) { 
    std::cerr << "\a\aNot updated for 3 years, might be OK.\n";
    r=OK;
    WY_HERE(r); std::cerr << Wy::wrd(r).c_str() << std::endl;
  }
 }

 // check that WyTimeSpec(0,0) relates to 1970-1-1 0000
 //
 {
   time_t tm_t(0);
   //struct tm* tsp=::localtime(&tm_t);
   struct tm* tsp=::gmtime(&tm_t);
   if(tsp==NULL) {
     WY_THROW( WyRet() ); 
   }
   
   try {
     if(tsp->tm_year!=70) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_mon!=0) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_mday!=1) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_hour!=0) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_min!=0) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_sec!=0) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_wday!=4) {
       WY_THROW( WyRet() ); 
     }
     if(tsp->tm_yday!=0) {
       WY_THROW( WyRet() ); 
     }
   }
   catch(...) {
     std::cerr << " -- WyTimeSpec(0,0) not relate to 1970-1-1 0000\n";
     dump_tm(tsp);
     throw;
   };
 }

 // Test set_systime
 //
 if(::getuid()==0) {
   // this is root
   WyTimeSpec tm0=Wy::now()-WyTimeSpec(Wy_Second(10),Wy_Nano());   // tm to set systime
   WyTimeSpec tm1;
   WyTimeSpec tm_now=Wy::now();

   if((r=Wy::set_systime(tm0))!=OK) {   // set time 10 secs earlier
     WY_THROW(r);
   }
   tm1=Wy::now();                       // read back
   if((r=Wy::set_systime(tm_now))!=OK) {// recover time
     WY_THROW(r);
   }
   if(((tm0-tm1)>WyTimeSpec(Wy_Second(),60000000))|| // diff>0.06sec
      ((tm0-tm1)<WyTimeSpec(Wy_Second(),-60000000))) { // diff>0.06sec
     std::cerr << Wy::wrd(tm0).c_str() << "!=" << Wy::wrd(tm1).c_str() << std::endl;
     WY_THROW( WyRet() ); 
   }

 } else {
   // non-superuser cannot set systeim time
   WyTimeSpec tm_now=Wy::now();
   if((r=Wy::set_systime(tm_now))!=Wym_EPERM) {
     WY_THROW(r);  
   }
 }

 // double/float conversion
 /*
 {
  WyTimeSpec tm("123.456e1");
  double a(tm);
  if((a<123.4559e1)||(a>123.4561e1)) {
    std::cerr << "tm= " << Wy::wrd(tm) << std::endl;
    std::cerr << "a= " << a << std::endl;
    WY_THROW( WyRet() );  
  }
  float b(tm);
  if((b<123.4559e1)||(b>123.4561e1)) {
    std::cerr << "tm= " << Wy::wrd(tm) << std::endl;
    std::cerr << "b= " << b << std::endl;
    WY_THROW( WyRet() );  
  }
 }*/

 // chk move ctor
 {
  char buf[2*sizeof(WyTimeSpec)];
  ::memset(buf,0,sizeof(buf));
  WyTimeSpec*p=reinterpret_cast<WyTimeSpec*>(buf);
  new(p) WyTimeSpec(23,4);
  new(p+1) WyTimeSpec(*p,Wy::ByMove);
  if(*(p+1)!=WyTimeSpec(23,4)) {
    (p+1)->~WyTimeSpec();
    WY_THROW( WyRet() );
  }
  (p+1)->~WyTimeSpec();
 }
};

/*
   Test errors (set_systime is not tested)
*/
static void t2(void)
{
 WyRet r;
 //
 // constructor Wym_ERANGE
 //
 try {
   WyTimeSpec ttm(MaxSecond,LongMax);   // should overflow
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // constructor Wym_ERANGE
 //
 try {
   WyTimeSpec ttm(MinSecond,LongMin);  // should underflow
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // constructor Wym_EFAULT
 //
 try {
   WyTimeSpec ttm((const char*)0);
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyCSeg::Reply& e) {
   if(e!=Wym_EFAULT) {
     WY_THROW( WyRet() );
   }
 };

 // constructor Wym_EFAULT
 //
 try {
   WyTimeSpec ttm(WyCSeg((const char*)0,size_t(0)));
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyCSeg::Reply& e) {
   if(e!=Wym_EFAULT) {
     WY_THROW( WyRet() );
   }
 };

 // ctor(double)
 //
 try {
   WyTimeSpec ttm(double(1e90));  // should overflow
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 try {
   WyTimeSpec ttm(double(-1e90));  // should overflow
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // Test reset(Wy_Second,Wy_Nano) overflow
 //
 {
  WyTimeSpec ttm;
  if((r=ttm.reset(MaxSecond,999999999L))!=OK) {  // this should be the max value
     WY_THROW(r);
  }
  if((r=ttm.reset(MaxSecond,1000000000L))!=Wym_ERANGE) { // this should overflow
     WY_THROW( WyRet() );
  }
  if(ttm!=WyTimeSpec(MaxSecond,999999999L)) {
     WY_THROW( WyRet() );     // error modified
  }
  if((r=ttm.reset(MinSecond,-999999999L))!=OK) {  // this should be the min value
     WY_THROW(r);
  }
  if((r=ttm.reset(MinSecond,-2000000001L))!=Wym_ERANGE) { // this should no be OK
     WY_THROW( WyRet() );
  }
 }

 // reset(const char*) error
 {
   WyTimeSpec t0("12.34");
   WyTimeSpec t1(t0);
   if((r=t1.reset(""))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if((r=t1.reset("abc"))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if(t1!=t0) {
     WY_THROW( WyRet() );
   }
   if((r=t1.reset("-0.1234567891"))!=OK) {
     WY_THROW(r);
   }
   if(t1!=WyTimeSpec("-0.123456789")) {  // fraction digits > 9 truncated
     WY_THROW( WyRet() );
   }
 }

 // reset(const char*,size_t) error
 {
   WyTimeSpec t0(WyCSeg("12.34",5));
   WyTimeSpec t1(t0);
   if((r=t1.reset(WyCSeg("",1)))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if((r=t1.reset("abc"))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if(t1!=t0) {
     WY_THROW( WyRet() );
   }
   if((r=t1.reset(WyCSeg("123456789123456789",18)))!=Wym_ERANGE) {
     WY_THROW(r);
   }
 }

 // operator=(const char*)
 //
 try {
   WyTimeSpec ttm;
   ttm="helo";
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ENOENT) {
     WY_THROW( WyRet() );
   }
 };

 // operator-()
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm=-ttm;
   if(ttm<=WyTimeSpec(MinSecond,0)) {
     WY_THROW( WyRet() ); 
   }
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_EMATHNEG) {
     WY_THROW( WyRet() );
   }
 };

 // operator++()
 //
 try {
   WyTimeSpec ttm(MaxSecond,0);
   ++ttm;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator--()
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   --ttm;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator++(int)
 //
 try {
   WyTimeSpec ttm(MaxSecond,0);
   ttm++;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator--(int)
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm--;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator+=(WyTimeSpec)
 //
 try {
   WyTimeSpec ttm(MaxSecond,0);
   ttm+=WyTimeSpec(1,1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm+=WyTimeSpec(-1,0);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator-=(WyTimeSpec)
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm-=WyTimeSpec(1,1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };
 try {
   WyTimeSpec ttm(MaxSecond,0);
   ttm-=WyTimeSpec(-1,0);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator+(WyTimeSpec)
 //
 try {
   WyTimeSpec ttm(MaxSecond,0);
   ttm=ttm+WyTimeSpec(1,1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // operator-(WyTimeSpec)
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm=ttm-WyTimeSpec(1,1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // abs()
 //
 try {
   WyTimeSpec ttm(MinSecond,0);
   ttm.abs();
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_EMATHNEG) {
     WY_THROW( WyRet() );
   }
 };

 // Test add(long ,long) overflow 
 //
 {
  WyTimeSpec ttm(MaxSecond,999999999L);   // maximum value
  WyRet r;
  if((r=ttm.add(Wy_Second(),Wy_Nano()))!=OK) {
     WY_THROW( WyRet() );
  }
  if((r=ttm.add(1,Wy_Nano()))!=Wym_ERANGE) {
     WY_THROW( WyRet() );
  }
  if(ttm!=WyTimeSpec(MaxSecond,999999999L)) {
     WY_THROW( WyRet() );     // error modified
  }
  if((r=ttm.add(Wy_Second(),1))!=Wym_ERANGE) {
     WY_THROW( WyRet() );
  }
  if(ttm!=WyTimeSpec(MaxSecond,999999999L)) {
     WY_THROW( WyRet() );     // error modified
  }

  {
    WyTimeSpec tmp(1,MaxNano);
    if(tmp.add(1,MaxNano)!=OK) {
      WY_THROW(r);
    }
    if(tmp.sub(1,MaxNano)!=OK) {
      WY_THROW(r);
    }
    if(tmp!=WyTimeSpec(1,MaxNano)) {
       WY_THROW( WyRet() );
    }
  }

  {
    WyTimeSpec tmp(1,MinNano);
    if(tmp.add(1,MinNano)!=OK) {
      WY_THROW(r);
    }
    if(tmp.sub(1,MinNano)!=OK) {
      WY_THROW(r);
    }
    if(tmp!=WyTimeSpec(1,MinNano)) {
       WY_THROW( WyRet() );
    }
  }
 }

 // Test add(WyTimeSpec) overflow
 //
 {
  WyTimeSpec ttm(MaxSecond,999999999L);   // maximum value
  WyRet r;
  if((r=ttm.add( WyTimeSpec(Wy_Second(),1) ))!=Wym_ERANGE) {
     WY_THROW(r);
  }
  if(ttm!=WyTimeSpec(MaxSecond,999999999L)) {
     WY_THROW( WyRet() );     // error modified
  }
 }

 // Test =(const char*)
 try {
   WyTimeSpec ttm(1,0);
   ttm="9a";
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_EBADMSG) {
     WY_THROW( WyRet() );
   }
 };

 //
 // Test +=WyTimeSpec overflow
 //
 try {
   WyTimeSpec ttm(MaxSecond,999999999L);
   ttm+=WyTimeSpec(Wy_Second(),1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 //
 // Test WyTimeSpec + WyTimeSpec overflow
 //
 try {
   WyTimeSpec ttm(MaxSecond,999999999L);
   ttm=ttm+WyTimeSpec(Wy_Second(),1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 //
 // Test WyTimeSpec - WyTimeSpec underflow
 //
 try {
   WyTimeSpec ttm(MinSecond,-999999999L);
   ttm=ttm-WyTimeSpec(Wy_Second(),1);
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 };

 // 
 // Test operator -() failure condition
 //
 try {
   WyTimeSpec ttm(MinSecond,Wy_Nano());   // negate MinSecond should fail
   ttm=-ttm;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_EMATHNEG) {
     WY_THROW( WyRet() );
   }
 };

 // 
 // Test operator -() failure condition
 //
 try {
   WyTimeSpec ttm(MinSecond,Wy_Nano());   // negate MinSecond should fail
   ttm=-ttm;
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_EMATHNEG) {
     WY_THROW( WyRet() );
   }
 };

 // don't know how to generate get_time/set_time error
};

// Test random time arithmatic operations
//
static void t3(void)
{
 WyRet r;
 WyTimeSpec t1,t2,t3,t4,tt;
 long i;
 for(i=100000L; i>0; --i) {
   t1=WyTimeSpec(Wy_Second(std::rand()%MaxL2),Wy_Nano(std::rand()));
   t2=WyTimeSpec(Wy_Second(std::rand()%MaxL2),Wy_Nano(std::rand()));
   t3=t1+t2;
   t4=t1-t2;
   tt=t3+t4;
   if(tt!=t1+t1) {
     WY_HERE(r);
     break;
   }
   tt=t4-t3;
   if(tt!=-t2-t2) {
     WY_HERE(r);
     break;
   }
 }
 if(i>0) { // premature break, error happened
   std::cout << " t1= " << Wy::wrd(t1).c_str()
             << "\n t2= " << Wy::wrd(t2).c_str()
             << "\n t3= " << Wy::wrd(t3).c_str()
             << "\n t4= " << Wy::wrd(t4).c_str()
             << "\n tt= " << Wy::wrd(tt).c_str()
             << std::endl;
   WY_THROW(r);
 }
};

//
//   Test the ensured range LLONG_MAX+0.999999999
//
static void t4(void) 
{
const WyTimeSpec WorkMax(MaxSecond,999999999L);
const WyTimeSpec WorkMin(-MaxSecond,-999999999L);
WyTimeSpec t1;

 //
 // Test -WorkMin
 //
 t1=-WorkMin;
 if(t1!=WorkMax) {
   WY_THROW( WyRet() );
 }

 //
 // Test -WorkMax
 //
 t1=-WorkMax;
 if(t1!=WorkMin) {
   WY_THROW( WyRet() );
 }

 //
 // Test WorkMax - 1nano
 //
 t1=WorkMax-WyTimeSpec(Wy_Second(),1);
 if(t1>=WorkMax) {
   WY_THROW( WyRet() );
 }
 t1+=WyTimeSpec(Wy_Second(),1);
 if(t1!=WorkMax) {
   WY_THROW( WyRet() );
 }

 //
 // Test WorkMax - 1sec
 //
 t1=WorkMax-WyTimeSpec(Wy_Second(1),Wy_Nano());
 if(t1>=WorkMax) {
   WY_THROW( WyRet() );
 }
 t1+=WyTimeSpec(1,Wy_Nano());
 if(t1!=WorkMax) {
   WY_THROW( WyRet() );
 }

 //
 // Test WorkMin + 1nano
 //
 t1=WorkMin+WyTimeSpec(Wy_Second(),1);
 if(t1<=WorkMin) {
   WY_THROW( WyRet() );
 }
 t1-=WyTimeSpec(Wy_Second(),1);
 if(t1!=WorkMin) {
   WY_THROW( WyRet() );
 }

 //
 // Test WorkMin + 1sec
 //
 t1=WorkMin+WyTimeSpec(1,Wy_Nano());
 if(t1<=WorkMin) {
   WY_THROW( WyRet() );
 }
 t1-=WyTimeSpec(1,Wy_Nano());
 if(t1!=WorkMin) {
   WY_THROW( WyRet() );
 }

 //
 // Constructor Wym_ERANGE
 //
 try {
   WyTimeSpec ttm(MaxSecond,LongMax);
   ttm.reset();   // supress compiler waring from above
   WY_THROW( WyRet() ); 
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 }
 catch(...) {
   std::cerr << "Exception Leaked" << std::endl;
 };
}

//
// Check sleep_till
//
#ifdef TEST_SLEEP_TILL
static void t5(void)
{
 WyTimeSpec t0=Wy::now();
 WyRet r=Wy::sleep_till(t0+WyTimeSpec(1,Wy_Nano())); // delay one second
 if(r!=Wym_ETIMEDOUT) {
   WY_THROW(r);
 }
 WyTimeSpec t1=Wy::now()-t0;
 if(  (t1<WyTimeSpec(Wy_Second(),Wy_Nano(940000000L)))      // 0.94 sec
    ||(t1>WyTimeSpec(Wy_Second(1),Wy_Nano(60000000L)))) {   // 1.06 sec
   std::cerr << "sleep_till(...) inaccurate delay, " << Wy::wrd(t1).c_str()
       << " secs (should be 1.0 second, may be OK)\n";
   //WY_THROW( WyRet() );    
 }
};
#endif

static void t_sleep(void)
{
 const WyTimeSpec OneSec(1,Wy_Nano());
 WyTimeSpec t0=Wy::now();
 WyRet r=Wy::sleep(OneSec);
 if(r!=OK) {
   WY_THROW(r);
 }
 WyTimeSpec t1=Wy::now()-t0;
 if(  (t1<WyTimeSpec(Wy_Second(),Wy_Nano(940000000L)))      // 0.94 sec
    ||(t1>WyTimeSpec(Wy_Second(1),Wy_Nano(60000000L)))) {   // 1.06 sec
   std::cerr << "sleep(..) inaccurate delay, " << Wy::wrd(t1).c_str()
       << " secs (should be 1.0 second, may be OK)\n";
   WY_THROW( WyRet() );  // error> 0.06 sec
 }

 WyTimeSpec trem(OneSec);
 t0=Wy::now();
 r=Wy::sleep(OneSec,trem);
 if(r!=OK) {
   WY_THROW(r);
 }
 t1=Wy::now()-t0;
 if(  (t1<WyTimeSpec(Wy_Second(),Wy_Nano(940000000L)))      // 0.94 sec
    ||(t1>WyTimeSpec(Wy_Second(1),Wy_Nano(60000000L)))) {   // 1.06 sec
   std::cerr << "sleep(..) inaccurate delay, " << Wy::wrd(t1).c_str()
       << " secs (should be 1.0 second, may be OK)\n";
   WY_THROW( WyRet() );  // error> 0.06 sec
 }
 if(trem!=OneSec) {
   // On Fedora3, success does not modify rem. Monter this feature.
   std::cerr << Wy::wrd(trem).c_str() << ".. feature changed\n";
   WY_THROW( WyRet() );
 }
};

// chk _mkstr
static void t_mkstr(void)
{
 struct ChkSmp {
   const char* nstr;
   int radix;
   size_t frdig;
   WyTimeSpec nvalue;
   void verify(void) const
     {
       WyStr str("before _mkstr called");
       WyRet r;
       try {
         if((r=Wy::_mkstr(str,nvalue,radix,frdig))!=OK) {
           WY_THROW(r);
         }
         if(str!=nstr) {
           WY_THROW( WyRet() );
         }
       }
       catch(...) {
         std::cerr << "ChkSmp={\"" << nstr << "\"," << radix << "," << frdig
              << ",WyTimeSpec(" << nvalue.second() << ","
              << nvalue.nano() << ")}, _mkstr=\""
              << str.c_str() << "\"\n";
         throw;
       };
     };
 } const chk_tab[]={
  {"0",10,0,WyTimeSpec()},{"0.0",10,1,WyTimeSpec()},{"0.00",10,2,WyTimeSpec()},
  {"1",10,0,WyTimeSpec(1,0)},{"-1",10,0,WyTimeSpec(-1,0)},{"1.0",10,1,WyTimeSpec(1,0)},
  {"26.000000501",10,9,WyTimeSpec(26,501)},
  {"21.000000054",10,9,WyTimeSpec(21,54)},
  {"1.00000",10,5,WyTimeSpec(1,5)},
  {"120.000000450",10,9,WyTimeSpec(120,450)},
  {"0.00000045",10,8,WyTimeSpec(Wy_Second(),450)},
  {"0",10,0,WyTimeSpec(Wy_Second(),450)},
  {"-0.00000045",10,8,WyTimeSpec(Wy_Second(),-450)},
  {"-1.00000045",10,8,WyTimeSpec(-1,-450)},
  {"-0.9999995500",10,10,WyTimeSpec(-1,450)},

  // radix=0
  {"0e+0",0,0,WyTimeSpec()},{"0.0e+0",0,1,WyTimeSpec()},
                            {"0.00e+0",0,2,WyTimeSpec()},
  {"1e+0",0,0,WyTimeSpec(1,0)},{"-1e+0",0,0,WyTimeSpec(-1,0)},
                            {"1.0e+0",0,1,WyTimeSpec(1,0)},
  {"2.6000000501e+1",0,10,WyTimeSpec(26,501)},
  {"2.1000000054e+1",0,10,WyTimeSpec(21,54)},
  {"1.00000e+0",0,5,WyTimeSpec(1,5)},
  {"1.2000000045e+2",0,10,WyTimeSpec(120,450)},
  {"4e-7",0,0,WyTimeSpec(Wy_Second(),450)},
  {"4.5e-7",0,1,WyTimeSpec(Wy_Second(),450)},
  {"4.50e-7",0,2,WyTimeSpec(Wy_Second(),450)},
  {"-4e-7",0,0,WyTimeSpec(Wy_Second(),-450)},
  {"-4.5e-7",0,1,WyTimeSpec(Wy_Second(),-450)},
  {"-4.50e-7",0,2,WyTimeSpec(Wy_Second(),-450)},
  {"-1.00000045e+0",0,8,WyTimeSpec(-1,-450)},
  {"-9.999995500e-1",0,9,WyTimeSpec(-1,450)},

 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
   chk_tab[i].verify();
 }

 WyRet r;
 WyStr str;
 if((r=Wy::_mkstr(str,WyTimeSpec(),1,0))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 if((r=Wy::_mkstr(str,WyTimeSpec(),2,1))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 if((r=Wy::_mkstr(str,WyTimeSpec(),8,1))!=Wym_EINVAL) {
   WY_THROW(r);
 }
 if((r=Wy::_mkstr(str,WyTimeSpec(),16,1))!=Wym_EINVAL) {
   WY_THROW(r);
 }
};

struct TestSample {
  const char* nstr;
  size_t strlen;
  WyTimeSpec nvalue;
};

static TestSample samp_tab[]= {
  {"0",1,WyTimeSpec()},{"+0",2,WyTimeSpec()},{"-0",2,WyTimeSpec()},
  {"00",2,WyTimeSpec()},{"+00",3,WyTimeSpec()},{"-00",3,WyTimeSpec()},
  {"0.0",3,WyTimeSpec()},{"+0.0",4,WyTimeSpec()},{"-0.0",4,WyTimeSpec()},
  {"0.00",4,WyTimeSpec()},{"+0.00",5,WyTimeSpec()},{"-0.00",5,WyTimeSpec()},
  {"00.00",5,WyTimeSpec()},{"+00.00",6,WyTimeSpec()},{"-00.00",6,WyTimeSpec()},
  {"0.",2,WyTimeSpec()},{"+0.",3,WyTimeSpec()},{"-0.",3,WyTimeSpec()},
  {".0",2,WyTimeSpec()},{"+.0",3,WyTimeSpec()},{"-.0",3,WyTimeSpec()},
  {"0e0",3,WyTimeSpec()},{"+0e0",4,WyTimeSpec()},{"-0e0",4,WyTimeSpec()},
  {"0e+0",4,WyTimeSpec()},{"+0e+0",5,WyTimeSpec()},{"-0e+0",5,WyTimeSpec()},
  {"0e-0",4,WyTimeSpec()},{"+0e-0",5,WyTimeSpec()},{"-0e00",5,WyTimeSpec()},

  {"12",2,WyTimeSpec(Wy_Second(12),Wy_Nano())},
  {"+12",3,WyTimeSpec(Wy_Second(12),Wy_Nano())},
  {"-12",3,WyTimeSpec(Wy_Second(-12),Wy_Nano())},

  {"12.34",5,WyTimeSpec(Wy_Second(12),Wy_Nano(340000000))},
  {"+12.34",6,WyTimeSpec(Wy_Second(12),Wy_Nano(340000000))},
  {"-12.34",6,WyTimeSpec(Wy_Second(-12),Wy_Nano(-340000000))},

  {"12.010",6,WyTimeSpec(Wy_Second(12),Wy_Nano(10000000))},
  {"+12.010",7,WyTimeSpec(Wy_Second(12),Wy_Nano(10000000))},
  {"-12.010",7,WyTimeSpec(Wy_Second(-12),Wy_Nano(-10000000))},

  {"012.34",6,WyTimeSpec(Wy_Second(12),Wy_Nano(340000000))},
  {"+012.34",7,WyTimeSpec(Wy_Second(12),Wy_Nano(340000000))},
  {"-012.34",7,WyTimeSpec(Wy_Second(-12),Wy_Nano(-340000000))},

  {"12.",3,WyTimeSpec(Wy_Second(12),Wy_Nano())},
  {"+12.",4,WyTimeSpec(Wy_Second(12),Wy_Nano())},
  {"-12.",4,WyTimeSpec(Wy_Second(-12),Wy_Nano())},

  {".12",3,WyTimeSpec(Wy_Second(),Wy_Nano(120000000))},
  {"+.12",4,WyTimeSpec(Wy_Second(),Wy_Nano(120000000))},
  {"-.12",4,WyTimeSpec(Wy_Second(),Wy_Nano(-120000000))},

  {".120",4,WyTimeSpec(Wy_Second(),Wy_Nano(120000000))},
  {"+.120",5,WyTimeSpec(Wy_Second(),Wy_Nano(120000000))},
  {"-.120",5,WyTimeSpec(Wy_Second(),Wy_Nano(-120000000))},

  {"12e1",4,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"12e+1",5,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"12e-1",5,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},

  {"+12e1",5,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"+12e+1",6,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"+12e-1",6,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},

  {"-12e1",5,WyTimeSpec(Wy_Second(-120),Wy_Nano())},
  {"-12e+1",6,WyTimeSpec(Wy_Second(-120),Wy_Nano())},
  {"-12e-1",6,WyTimeSpec(Wy_Second(-1),Wy_Nano(-200000000))},

  {"12.21e1",7,WyTimeSpec(Wy_Second(122),Wy_Nano(100000000))},
  {"12.21e+1",8,WyTimeSpec(Wy_Second(122),Wy_Nano(100000000))},
  {"12.21e-1",8,WyTimeSpec(Wy_Second(1),Wy_Nano(221000000))},

  {"12.010e1",8,WyTimeSpec(Wy_Second(120),Wy_Nano(100000000))},
  {"12.010e+1",9,WyTimeSpec(Wy_Second(120),Wy_Nano(100000000))},
  {"12.010e-1",9,WyTimeSpec(Wy_Second(1),Wy_Nano(201000000))},

  {"12.e1",5,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"12.e+1",6,WyTimeSpec(Wy_Second(120),Wy_Nano())},
  {"12.e-1",6,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},

  {".12e1",5,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},
  {".12e+1",6,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},
  {".12e-1",6,WyTimeSpec(Wy_Second(),Wy_Nano(12000000))},

  {".120E1",6,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},
  {".120E+1",7,WyTimeSpec(Wy_Second(1),Wy_Nano(200000000))},
  {".120E-1",7,WyTimeSpec(Wy_Second(),Wy_Nano(12000000))},

  // regular use (positive)
  {"123e-3",6,WyTimeSpec(Wy_Second(),Wy_Nano(123000000))},
  {"123e-6",6,WyTimeSpec(Wy_Second(),Wy_Nano(123000))},
  {"123e-9",6,WyTimeSpec(Wy_Second(),Wy_Nano(123))},
  {"1e-3",4,WyTimeSpec(Wy_Second(),Wy_Nano(1000000))},
  {"1e-6",4,WyTimeSpec(Wy_Second(),Wy_Nano(1000))},
  {"1e-9",4,WyTimeSpec(Wy_Second(),Wy_Nano(1))},
  {"345.678e3",9,WyTimeSpec(Wy_Second(345678),Wy_Nano())},

  // regular use (negative)
  {"-123e-3",7,WyTimeSpec(Wy_Second(),Wy_Nano(-123000000))},
  {"-123e-6",7,WyTimeSpec(Wy_Second(),Wy_Nano(-123000))},
  {"-123e-9",7,WyTimeSpec(Wy_Second(),Wy_Nano(-123))},
  {"-1e-3",5,WyTimeSpec(Wy_Second(),Wy_Nano(-1000000))},
  {"-1e-6",5,WyTimeSpec(Wy_Second(),Wy_Nano(-1000))},
  {"-1e-9",5,WyTimeSpec(Wy_Second(),Wy_Nano(-1))},
  {"-345.678e3",10,WyTimeSpec(Wy_Second(-345678),Wy_Nano())},

  // larger number (+/- LONG_INT)
  {"2147483647",10,WyTimeSpec(Wy_Second(2147483647L),Wy_Nano())},
  {"2147483647.0",12,WyTimeSpec(Wy_Second(2147483647L),Wy_Nano())},
  {"2147483647.0e-1",15,WyTimeSpec(Wy_Second(214748364L),Wy_Nano(700000000))},
  {"2147483647.0e-9",15,WyTimeSpec(Wy_Second(2),Wy_Nano(147483647))},
  {"2.1474836470e9",14,WyTimeSpec(Wy_Second(2147483647L),Wy_Nano())},

  // digits less than 1/billion are truncated
  {"2147483647.0e-10",16,WyTimeSpec(Wy_Second(),Wy_Nano(214748364))},
  {"2147483647.0e-11",16,WyTimeSpec(Wy_Second(),Wy_Nano(21474836))},
  {"12.3456789e-2",13,WyTimeSpec(Wy_Second(),Wy_Nano(123456789))},
  {"12.3456789e-9",13,WyTimeSpec(Wy_Second(),Wy_Nano(12))},
  {"12.123456789",12,WyTimeSpec(Wy_Second(12),Wy_Nano(123456789))},
  {"12.123456789123",15,WyTimeSpec(Wy_Second(12),Wy_Nano(123456789))},
};

static void verify_sample(const TestSample& sample)
{
 WyRet r;
 WyTimeSpec res;
 const char* endptr;

 try {
   if(sample.nstr==0) {
     WY_THROW(r);
   }
   if((r=Wy::_strnum(res,&endptr,WyCSeg(sample.nstr,sample.strlen)))!=OK) {
     WY_THROW(r);
   }
    if(endptr!=sample.nstr+sample.strlen) {
      WY_THROW( WyRet() );
    }
    if(res!=sample.nvalue) {
      WY_THROW( WyRet() );
    }
 }
 catch(...) {
   std::cerr << "TestSample={\"" << sample.nstr << "\"," << sample.strlen
             << ",WyTimeSample(" << sample.nvalue.second() << ","
             << sample.nvalue.nano() << ")}\n";
   throw;
 }
};

static void t_strnum_samples(void)
{
 for(size_t i=0; i<sizeof(samp_tab)/sizeof(TestSample); ++i) {
   verify_sample(samp_tab[i]);
 }
};

static void t_strnum_error(void)
{
 typedef WyTimeSpec ResType;
 const ResType Org("56.8923");
 const char* endptr;
 ResType res;
 WyRet r;

 // chk errors
 {
  res=Org;
  try {
    r=Wy::_strnum(res,&endptr,(const char*)0);
    WY_THROW(r);
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

  if(res!=Org) {
    WY_THROW( WyRet() );
  }
  if((r=Wy::_strnum(res,&endptr,WyCSeg("",(size_t)0)))!=Wym_ENOENT) {
    WY_THROW(r);
  }
  if(res!=Org) {
    WY_THROW( WyRet() );
  }
  {
    const char es1[]="233aa";
    const char es2[]="+a";
    const char es3[]="-a";
    const char es4[]="+-.23";
    const char es5[]="1.23xa";
    
    res=Org;
    if((r=Wy::_strnum(res,&endptr,WyCSeg(es1,sizeof(es1)-1)))!=Wym_EBADMSG) {
      WY_THROW(r);
    }
    if(endptr!=es1+3) {
      WY_THROW( WyRet() );
    }
    if(res!=Org) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_strnum(res,&endptr,WyCSeg(es2,sizeof(es2)-1)))!=Wym_ENOENT) {
      WY_THROW(r);
    }
    if(endptr!=es2+1) {
      WY_THROW( WyRet() );
    }
    if(res!=Org) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_strnum(res,&endptr,WyCSeg(es3,sizeof(es3)-1)))!=Wym_ENOENT) {
      WY_THROW(r);
    }
    if(endptr!=es3+1) {
      WY_THROW( WyRet() );
    }
    if(res!=Org) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_strnum(res,&endptr,WyCSeg(es4,sizeof(es4)-1)))!=Wym_ENOENT) {
      WY_THROW(r);
    }
    if(endptr!=es4+1) {
      WY_THROW( WyRet() );
    }
    if(res!=Org) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_strnum(res,&endptr,WyCSeg(es5,sizeof(es5)-1)))!=Wym_EBADMSG) {
      WY_THROW(r);
    }
    if(endptr!=es5+4) {
      WY_THROW( WyRet() );
    }
    if(res!=Org) {
      WY_THROW( WyRet() );
    }
  }
 }
 
 {
  const char tstr[]="12.3e1X";
  res=Org;
  if((r=Wy::_strnum(res,&endptr,WyCSeg(tstr,sizeof(tstr)-1)))!=Wym_EBADMSG) {
    WY_THROW(r);
  }
  if(endptr!=tstr+6) {
    WY_THROW( WyRet() );
  }
  if(res!=Org) {
    WY_THROW( WyRet() );
  }
 }

 {
  const char tstr[]="12.3e+-1";
  res=Org;
  if((r=Wy::_strnum(res,&endptr,WyCSeg(tstr,sizeof(tstr)-1)))!=Wym_EBADMSG) {
    WY_THROW(r);
  }
  if(endptr!=tstr+6) {
    WY_THROW( WyRet() );
  }
  if(res!=Org) {
    WY_THROW( WyRet() );
  }
 }

 {
  const char tstr[]="12.3e20";
  res=Org;
  if((r=Wy::_strnum(res,&endptr,WyCSeg(tstr,sizeof(tstr)-1)))!=Wym_ERANGE) {
    WY_THROW(r);
  }
  if(endptr!=tstr+sizeof(tstr)-1) {
    WY_THROW( WyRet() );
  }
  if(res!=Org) {
    WY_THROW( WyRet() );
  }
 }

 {
  const char tstr[]="12.3e-20";
  res=Org;
  if((r=Wy::_strnum(res,&endptr,WyCSeg(tstr,sizeof(tstr)-1)))!=Wym_ERANGE) {
    WY_THROW(r);
  }
  if(endptr!=tstr+sizeof(tstr)-1) {
    WY_THROW( WyRet() );
  }
  if(res!=Org) {
    WY_THROW( WyRet() );
  }
 }

 {
  const char s1[]="01234567890";
  const char s2[]="0.12345678900001";

  if((r=Wy::_strnum(res,&endptr,WyCSeg(s1,11)))!=OK) {
    WY_THROW(r);
  }
  if(endptr!=s1+sizeof(s1)-1) {
    WY_THROW( WyRet() );
  }
  if((r=Wy::_strnum(res,&endptr,WyCSeg(s2,11)))!=OK) {
    WY_THROW(r);
  }
  if((r=Wy::_strnum(res,&endptr,WyCSeg(s2,12)))!=OK) {
    WY_THROW(r);
  }
  //if(endptr!=s2+sizeof(s2)-1) {  _strnum for WyTimeSpec may return OK
  //  WY_THROW( WyRet() );         while endptr in the middle of input string
  //}

  {
    ResType tmp;
    if((r=Wy::_strnum(res,&endptr,WyCSeg(s2,15)))!=OK) {
      WY_THROW(r);
    }
    if((r=Wy::_strnum(tmp,&endptr,WyCSeg(s2,16)))!=OK) {
      WY_THROW(r);
    }
    if(res!=tmp) {
      WY_THROW( WyRet() );
    }
  }
 }

 // chk error max
 {
   WyTimeSpec tmp(WyTimeSpec::max()); 
   tmp+=WyTimeSpec();
   tmp-=WyTimeSpec();
 }
 try {
   WyTimeSpec tmp(WyTimeSpec::max());
   tmp+=WyTimeSpec(Wy_Second(),Wy_Nano(1));
   WY_THROW( WyRet() );
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     throw;
   }
 };
 // chk error min
 {
   WyTimeSpec tmp(WyTimeSpec::min()); 
   tmp-=WyTimeSpec();
   tmp+=WyTimeSpec();
 }
 try {
   WyTimeSpec tmp(WyTimeSpec::min());
   tmp-=WyTimeSpec(Wy_Second(),Wy_Nano(1));
   WY_THROW( WyRet() );
 }
 catch(const WyTimeSpec::Reply& e) {
   if(e!=Wym_ERANGE) {
     throw;
   }
 };

};

static void t_wrd(void)
{
 struct ChkSmp {
   WyTimeSpec tsp;
   const char* nstr;
   void verify(void) const
     {
      if(Wy::wrd(tsp)!=nstr) {
        std::cerr << "WyTimeSpec(" << tsp.second() << "," << tsp.nano()
                  << "), \"" << nstr << "\", wrd()=" 
                  << Wy::wrd(tsp).c_str() << std::endl;
        WY_THROW( WyRet() ); 
      }
     };
 } const chk_tab[]={
    {WyTimeSpec(Wy_Second(),Wy_Nano()),"0.0"},
    {WyTimeSpec(),"0.0"},
    {WyTimeSpec(Wy_Second(),Wy_Nano(1230)),"0.00000123"},
    {WyTimeSpec(Wy_Second(),Wy_Nano(-1230)),"-0.00000123"},
    {WyTimeSpec(Wy_Second(1),Wy_Nano(1230)),"1.00000123"},
    {WyTimeSpec(Wy_Second(1),Wy_Nano(-1230)),"0.99999877"},
    {WyTimeSpec(Wy_Second(-1),Wy_Nano(1230)),"-0.99999877"},
    {WyTimeSpec(Wy_Second(-1),Wy_Nano(-1230)),"-1.00000123"},
    {WyTimeSpec(Wy_Second(1),Wy_Nano(-1)),"0.999999999"},
    {WyTimeSpec(120,450),"120.00000045"},
    {WyTimeSpec(Wy_Second(),450),"0.00000045"},
    {WyTimeSpec(Wy_Second(),-450),"-0.00000045"},
    {WyTimeSpec(-1,-450),"-1.00000045"},
    {WyTimeSpec(-1,450),"-0.99999955"},
    {WyTimeSpec("120.45e-1"),"12.045"},
 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
   chk_tab[i].verify();
 }

 // check wd(WyTimeSpec,int,size_t)
 {
  WyTimeSpec ts(123,987654321);
  if(Wy::wrd(ts,10,3)!="123.987") {
    WY_THROW( WyRet() );
  }
  if(Wy::wrd(ts, 0,3)!="1.239e+2") {
    WY_THROW( WyRet() );
  }
  
  // chk error
  try {
    Wy::wrd(ts,16,3);
    WY_THROW( WyRet() );
  }
  catch(const WyStr::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet(e) );
    }
    // FALLTHROUGH
  }
 }
};

#include "wy_array.h"
#include "ck_array.h"

static void t_array(void)
{
 /*
 // chk struct timespec { time_t tv_sec; long tv_nsec; }
 // (simple chk)
 {
   typedef ::timespec ElemType;
   Wy_Array<ElemType> tmp;
   ::timespec ts={ 123,456 };
   tmp.push_back(ts);
   if(tmp.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].tv_sec!=ts.tv_sec) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].tv_nsec!=ts.tv_nsec) {
     WY_THROW( WyRet() );
   }

   tmp.push_back(ElemType());
   if(tmp.size()!=2) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].tv_sec!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].tv_nsec!=0) {
     WY_THROW( WyRet() );
   }
 }*/

 /*
 // chk struct timeval { time_t tv_sec; long tv_usec; }
 // (simple check)
 {
   typedef ::timeval ElemType;
   Wy_Array<ElemType> tmp;
   ::timeval ts={ 123,456 };
   tmp.push_back(ts);
   if(tmp.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].tv_sec!=ts.tv_sec) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].tv_usec!=ts.tv_usec) {
     WY_THROW( WyRet() );
   }

   tmp.push_back(ElemType());
   if(tmp.size()!=2) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].tv_sec!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].tv_usec!=0) {
     WY_THROW( WyRet() );
   }
 }*/
};

void ck_timespec(void)
{
 t1();
 t2();
 t3();
 t4();
#ifdef TEST_SLEEP_TILL
 t5();
#endif
 t_sleep();
 t_mkstr();
 t_strnum_samples();
 t_strnum_error();
 t_wrd();
 t_array();
};
