/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

   Basci conversions between ascii-string and integral type
*/
#ifndef WY__NUM_H__
#define WY__NUM_H__

#include "wyret.h"
#include "wycseg.h"
#include <limits>

namespace Wy {

 inline int _charnum(int ch) WY__TSPC()
 {
  if(ch<='9') {
    if(ch<'0') {
      return(-1);
    }
    return(ch-'0');
  } else if(ch>='a') {
    if(ch>'z') {
      return(-1);
    }
    return(ch-'a'+int(10));
  } else if(ch>='A') {
    if(ch>'Z') {
      return(-1);
    }
    return(ch-'A'+int(10));
  } else {
    return(-1);
  }
 };

// Declaration of _strnum of all supported built-in types:
//  char, 
//  signed(char, short, int, long , long long)
//  unsigned(char, short, int, long , long long)
//  float, double
/*
WyRet _strnum(char& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(signed char& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(signed short& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(signed int& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(signed long& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(signed long long& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(unsigned char& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(unsigned short& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(unsigned int& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(unsigned long& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(unsigned long long& res, const char** endptr, const char* buf, size_t blen, const int& radix);
WyRet _strnum(float& res, const char** endptr, const char* buf,size_t blen, const int& radix);
WyRet _strnum(double& res, const char** endptr, const char* buf,size_t blen, const int& radix);
*/
WyRet _strnum(char& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(signed char& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(signed short& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(signed int& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(signed long& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(signed long long& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(unsigned char& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(unsigned short& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(unsigned int& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(unsigned long& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(unsigned long long& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(float& res, const char** endptr, WyCSeg cseg, const int& radix);
WyRet _strnum(double& res, const char** endptr, WyCSeg cseg, const int& radix);

//
/*
WyRet _strnum(char& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(signed char& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(signed short& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(signed int& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(signed long& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(signed long long& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(unsigned char& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(unsigned short& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(unsigned int& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(unsigned long& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(unsigned long long& res, const char** endptr, const char* buf, size_t blen, int& radix);
WyRet _strnum(float& res, const char** endptr, const char* buf,size_t blen, int& radix);
WyRet _strnum(double& res, const char** endptr, const char* buf,size_t blen, int& radix);
*/
WyRet _strnum(char& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(signed char& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(signed short& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(signed int& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(signed long& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(signed long long& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(unsigned char& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(unsigned short& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(unsigned int& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(unsigned long& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(unsigned long long& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(float& res, const char** endptr, WyCSeg cseg, int& radix);
WyRet _strnum(double& res, const char** endptr, WyCSeg cseg, int& radix);

inline WyRet _strnum(char& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(signed char& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(signed short& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(signed int& res, const char** endptr, const WyCSeg& cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(signed long& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(signed long long& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(unsigned char& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(unsigned short& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(unsigned int& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(unsigned long& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
inline WyRet _strnum(unsigned long long& res, const char** endptr, WyCSeg cseg)
{ return _strnum(res,endptr,cseg,10); };
WyRet _strnum(float& res, const char** endptr, WyCSeg cseg);
WyRet _strnum(double& res, const char** endptr, WyCSeg cseg);

//
WyRet wy_strnum2(signed int& res1, signed int& res2, int& fdig, const char** endptr, const char* buf,size_t blen);
WyRet wy_strnum2(signed long& res1, signed long& res2, int& fdig, const char** endptr, const char* buf,size_t blen);
WyRet wy_strnum2(signed long long& res1, signed long long& res2, int& fdig, const char** endptr, const char* buf,size_t blen);

WyRet wy_strnum3(signed int& res1, signed int& res2, int& fdig, int& res3, const char** endptr, const char* buf,size_t blen);
WyRet wy_strnum3(signed long& res1, signed long& res2, int& fdig, int& res3, const char** endptr, const char* buf,size_t blen);
WyRet wy_strnum3(signed long long& res1, signed long long& res2, int& fdig, int& res3, const char** endptr, const char* buf,size_t blen);

// [Syn] Range checked multiplication
//       The result of n1*n2 is stored into res
//
// [Ret] OK
//       Wym_ERANGE
//
template <typename NumT>
WyRet wy_mul(NumT& res, NumT n1, NumT n2)
{
 if(std::numeric_limits<NumT>::is_exact) {
   const NumT tmp(n1*n2);
   if(n1!=0) {
     if((tmp/n1)!=n2) {
       WY_RETURN(Wym_ERANGE);
     }
   } else if(n2!=0) {
     if((tmp/n2)!=n1) {
       WY_RETURN(Wym_ERANGE);
     }
   } else {};
   res=tmp;
 } else {
   WY_TERMINATE("");   // in case
 }
 return(OK);
};

// [Syn] Range checked addition
//       The result of n1+n2 is stored into res
//
// [Ret] OK
//       Wym_ERANGE
//
template <typename NumT>
WyRet wy_add(NumT& res, NumT n1, NumT n2)
{
 if(std::numeric_limits<NumT>::is_signed) {
   const NumT tmp(n1+n2);
   if(n1<=NumT(0)) {
     if(tmp>n2) {
       WY_RETURN(Wym_ERANGE);
     }
   } else {
     if(tmp<=n2) {
       WY_RETURN(Wym_ERANGE);
     }
   }
   res=tmp;
 } else {
   const NumT tmp(n1+n2);
   if(tmp<n2) {
     WY_RETURN(Wym_ERANGE);
   }
   res=tmp;
 }
 return(OK);
};

}; // End of Wy

#endif // End of Wy__Num
