/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   This header file is included in wy_uty.h

   Documentation: wy__rdque.3wy
*/
#ifndef WY__RDQUE_H__
#define WY__RDQUE_H__
#include "wybyteflow.h"

// One purpose of Wy__RdQue is for testing usibility of this library, because
// many existing applications read/write in the streaming I/O way, particularly
// I/O redirection is involved.
//
// Members getdata(..) return front data segment in the internal queue buffer
// (comparing with return WyStr) for this is privitive, and data copy is
// usually the major performance factor. (no ungetc like functionality)
//
// Known problem: 
//       WyByteFlow regf("some_regular_file",O_RDONLY);
//
//       void f(WyByteFlow& bf) {
//         Wy__RdQue rdq(bf);
//         dynamic_cast<WyRegFile&>(bf).set_pos(0);
//         WyByteFlow tmp(bf);
//         tmp.read(...);       // read pos could be random (not specified)
//       }
//
class Wy__RdQue {
    WyByteFlow* _fptr;  // pointer of the WyByteFlow object to read data from
    WyStr _rqbuf;       // read queue buffer
    size_t _cidx;       // starting index for the next getdata output
    size_t _dtsize;     // data transfer size ('count' for read)

    WyRet _init_dtsize(const WyByteFlow& bf);
    static void _set_dtsize(Wy__RdQue& rdq,size_t size); // For Wy_AtDestroy

    // Hidden
    Wy__RdQue(const Wy__RdQue& bf);
    const Wy__RdQue& operator=(const Wy__RdQue& rhs);
    bool operator==(const Wy__RdQue& rhs) const;
    bool operator!=(const Wy__RdQue& rhs) const;
   
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    explicit  // no implicit conversion
    Wy__RdQue(WyByteFlow& bf);

    ~Wy__RdQue() WY__TSPC();

    WyByteFlow& rfile(void) const WY__TSPC();

    WyCSeg qdata(void) const WY__TSPC();

    WyRet reset(WyByteFlow&);

    WyRet getdata(char& chr);

    WyRet getdata(WyCSeg& peek, size_t count);

    WyRet getdata(WyCSeg& peek, size_t count, char c);

    size_t _rdsize(void) const WY__TSPC();
    void _rdsize(size_t) WY__TSPC();
    WyRet _feedque(size_t& n_read);
};

#endif
