/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WY_LNKFILE_H__
#define WY_LNKFILE_H__
#define WY_LNKFILE_VERSION 30

#include "wyfilestat.h"
#include "wyfilehandle.h"

/*
  Wy_LnkFile defines the class for the system link file.

  Note: The open object is implemented just by recording the pathname.
        There is no file descriptor for linkfile
*/
class Wy_LnkFile {
    // Hidden
    const Wy_LnkFile & operator =(const Wy_LnkFile &);
    bool operator==(const Wy_LnkFile&) const;
    bool operator!=(const Wy_LnkFile&) const;

    WyStr _pname;

  public:
    static const char class_name[];
    WY_THROW_REPLY;

    Wy_LnkFile();
    Wy_LnkFile(const Wy_LnkFile &src);
    explicit  // no implicit conversion
    Wy_LnkFile(const char* pathname);
    explicit  // no implicit conversion
    Wy_LnkFile(const WyStr& pathname);

    // [Move Constructor]
    //
    // Note: src must be whole object except in initialization list
    //
    Wy_LnkFile(Wy_LnkFile &src, Wy::ByMove_t) WY__TSPC();

    virtual ~Wy_LnkFile();

    bool is_default(void) const WY__TSPC();

    WyRet stat(WyFileStat& filestat) const;

    virtual WyRet reset(void);
    virtual WyRet reset(const Wy_LnkFile& src);
    virtual WyRet reset(const char* pathname);
    virtual WyRet reset(const WyStr& pathname);

    virtual WyRet _swap(Wy_LnkFile& f) WY__TSPC();
    virtual Wy_LnkFile* _alloc(WyRet&) const;

    virtual WyRet read(WyStr& value);

    static WyRet create(const char* oldpath,const char* newpath);
    static WyRet create(const WyStr& oldpath,const WyStr& newpath);
    static WyRet create(const char* oldpath,const WyStr& newpath);
    static WyRet create(const WyStr& oldpath,const char* newpath);

    const WyStr& wy_lnkf(void) const WY__NOTHROW__
             { return _pname; };
};

#endif
