/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

  WyDate deals with the calendar notion of year,month,day,hour,minute,second.

*/
#ifndef WYDATE_H__
#define WYDATE_H__

#define WYDATE_VERSION 30

#include "wytimespec.h"
#include "wystr.h"

/*
  [Converted] struct tm,
            mktime(...), localtime_r(...);
*/
class WyDate {
    class Impl;
  public:
    static const char class_name[];
    static const char TIMEZONE_UTC[];
    WY_THROW_REPLY;  // declaraion of Reply

    WyDate() WY__TSPC(Reply);
    WyDate(const WyDate& src) WY__TSPC(Reply);
    WyDate(const char* tzval,const WyTimeSpec& ucs);
    WyDate(const WyStr& tzval,const WyTimeSpec& ucs);
    WyDate(const char* tzval,int year,int month,int mday, int hour,int min, int sec);
    WyDate(const WyStr& tzval,int year,int month,int mday, int hour,int min, int sec);

    WyDate(WyDate& src, Wy::ByMove_t) WY__TSPC();
    ~WyDate() WY__TSPC();
 
    bool is_default(void) const WY__TSPC();
    
    void reset(void) WY__TSPC();
    WyRet reset(const WyDate& rhs) WY__TSPC();
    WyRet reset(const WyStr& tzval,const WyTimeSpec& ucs);
    WyRet reset(const char* tzval,const WyTimeSpec& ucs);
    WyRet reset(const WyStr&,int year,int month,int mday,
                 int hour,int min, int sec);
    WyRet reset(const char*,int year,int month,int mday,
                 int hour,int min, int sec);
    void swap(WyDate& ano) WY__TSPC();

    const WyStr& time_zone(void) const WY__TSPC();

    int year(void) const WY__TSPC();

    int month(void) const WY__TSPC();

    int mday(void) const WY__TSPC();

    int hour(void) const WY__TSPC();

    int min(void) const WY__TSPC();

    int sec(void) const WY__TSPC();

    int wday(void) const WY__TSPC();

    int yday(void) const WY__TSPC();

    int isdst(void) const WY__TSPC();

    WyTimeSpec utc_time(void) const WY__TSPC();

    WyRet set_utc(const WyTimeSpec& tm);

    WyRet set_date(int year,int month,int mday, int hour,int min, int sec);

    WyRet add(const WyTimeSpec& tm);

    const WyDate& operator =(const WyDate& rhs) WY__TSPC(Reply);

    const WyDate& operator +=(const WyTimeSpec& rhs);

    bool operator ==(const WyDate&) const WY__TSPC();
    bool operator !=(const WyDate&) const WY__TSPC();

    const Impl* wy_impl(void) const WY__NOTHROW__
           { return _dptr; };   
    Impl* wy_impl(void) WY__NOTHROW__
           { return _dptr; };   
  private:
    static const int Default_Year=1970;
    static const int Default_Month=1;
    static const int Default_MDay=1;
    static const int Default_Hour=0;
    static const int Default_Min=0;
    static const int Default_Sec=0;
    static const int Default_WDay=4;
    static const int Default_YDay=0;

    Impl *_dptr;
};

WyDate operator +(const WyDate& dt,const WyTimeSpec& tm);
WyDate operator +(const WyTimeSpec& tm,const WyDate& dt);

namespace Wy {
 WyStr wrd(const WyDate&);
 WyRet _strftime(WyStr&, const char*, const WyDate&);
 WyStr local_tzname(void);
};

#endif
