/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WYFILESTAT_H__
#define WYFILESTAT_H__
#define WYFILESTAT_VERSION 30

#include "wytimespec.h"
#include <sys/stat.h>

/*
   WyFileStat is the class for "C" struct stat

  [Converted] struct ::stat
*/
class WyFileStat {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyFileStat() WY__TSPC();
    WyFileStat(const WyFileStat &src) WY__TSPC();
    WyFileStat(WyFileStat &src, Wy::ByMove_t) WY__TSPC();
    WyFileStat(const struct stat&) WY__TSPC();

    bool is_default(void) const WY__TSPC();

    void reset(void) WY__TSPC();
    void reset(const WyFileStat& src) WY__TSPC();

    void swap(WyFileStat& src) WY__TSPC();

    const WyFileStat& operator =(const WyFileStat &rhs) WY__TSPC();

    mode_t mode(void) const WY__TSPC();
    ino_t ino(void) const WY__TSPC();
    dev_t dev(void) const WY__TSPC();
    nlink_t nlink(void) const WY__TSPC();
    uid_t uid(void) const WY__TSPC();
    gid_t gid(void) const WY__TSPC();
    dev_t _rdev(void) const WY__TSPC();
    off_t size(void) const WY__TSPC();
    blksize_t _blksize(void) const WY__TSPC();
    blkcnt_t _blocks(void) const WY__TSPC();
    Wy_Second atime(void) const WY__TSPC();
    Wy_Second ctime(void) const WY__TSPC();
    Wy_Second mtime(void) const WY__TSPC();

    bool is_dir(void) const WY__TSPC();
    bool is_chr(void) const WY__TSPC();
    bool is_blk(void) const WY__TSPC();
    bool is_reg(void) const WY__TSPC();
    bool is_fifo(void) const WY__TSPC();
    bool is_lnk(void) const WY__TSPC();
    bool is_sock(void) const WY__TSPC();

    bool operator ==(const WyFileStat& rhs) const WY__TSPC();
    bool operator !=(const WyFileStat& rhs) const WY__TSPC();

    // [Internal] Get the address of struct stat in use
    //
    // [Ret] Address of struct stat in use (never 0)
    //
    struct stat* wy_stat_ptr(void) WY__NOTHROW__;

  private:
    struct stat _stt;
};

namespace Wy {
 WyRet stat(const char*,WyFileStat& stt);
 WyRet stat(const WyStr&,WyFileStat& stt);
 WyRet lstat(const char*,WyFileStat& stt);
 WyRet lstat(const WyStr&,WyFileStat& stt);
};

/*
// Wy_Array specilization to support struct timespec,timeval
//
#include "wy_array.h"
template<>
struct Wy_Array< struct stat > : Wy__ArrayValue< struct stat > {
  Wy_Array() : Wy__ArrayValue< struct stat >() {};
  Wy_Array(size_t s,const struct stat& elem) : Wy__ArrayValue< struct stat >(s,elem) {};
  Wy_Array(const Wy_Array& s) : Wy__ArrayValue< struct stat >(s) {};
  Wy_Array(Wy_Array& p, Wy::ByMove_t t) : Wy__ArrayValue< struct stat >(p,t) {};
};
*/

#endif
