/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Documentation in file wyterminal.3wy
*/
#ifndef WYTERMINAL_H__
#define WYTERMINAL_H__
#define WYTERMINAL_VERSION 30

#include "wychrfile.h"
#include "wytermios.h"

// This function may be cancellation point on different systems
//
int wyc_ttyname(int fd, char* buf, size_t blen);

class WyRegFile;
class WyChrFile;
class WyTerminal;
class WyFifoFile;
class WySockFile;

class WyTerminal : public WyChrFile {
    // Hidden
    const WyTerminal & operator =(const WyTerminal&);
    bool operator==(const WyTerminal&) const ;
    bool operator!=(const WyTerminal&) const ;
    mode_t ftype(void) const WY__TSPC();

    // Hidden
    WyRet _tcdrain(void);
    bool _isatty(void) const;

    // Hidden (for compile time rule out)
    WyTerminal(const WyRegFile&);
    //WyTerminal(const WyChrFile&);
    //WyTerminal(const WyTerminal&);
    WyTerminal(const WyFifoFile&);
    WyTerminal(const WySockFile&);
    WyRet reset(const WyRegFile&);
    //WyRet reset(const WyChrFile&);
    //WyRet reset(const WyTerminal&);
    WyRet reset(const WyFifoFile&);
    WyRet reset(const WySockFile&);
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyTerminal();
    explicit  // no implicit conversion
    WyTerminal(WyFileHandle fh);
    WyTerminal(const WySysFile& termf);
    WyTerminal(const char* pathname,int f);
    WyTerminal(const WyStr& pathname,int f);

    WyTerminal(const WyTerminal& termf);

    // [Move Constructor]
    //
    // Note: termf must be whole object except in initialization list
    //
    WyTerminal(WyTerminal& termf, Wy::ByMove_t) WY__TSPC();

    //virtual ~WyTerminal() ;

    /* [Derived]
    bool is_default(void) const WY__TSPC();
    WyFileHandle fh(void) const WY__TSPC();
    WyRet stat(WyFileStat& filestat) const;
    */

    // [Override]
    virtual WyRet reset(void)
         { return WyChrFile::reset(); };
    virtual WyRet reset(WyFileHandle fh);
    virtual WyRet reset(const WySysFile& termf);
    virtual WyRet reset(const char* pathname,int f);
    virtual WyRet reset(const WyStr& pathname,int f);

    // [Derived]
    //virtual WyRet _swap(WySysFile& src) WY__TSPC();

    //[Override]
    virtual WyTerminal* _alloc(WyRet& r) const;

    // [Derived]
    //virtual WyRet read(void *, size_t, size_t&);
    //virtual WyRet read(WyStr&, size_t, size_t&);
    //virtual WyRet _xread(void*, size_t, size_t&);

    //virtual WyRet write(const void *, size_t, size_t&);
    //virtual WyRet write(WyCSeg, size_t&);
    //virtual WyRet write(const WyStr&, size_t&);

    // [Added]
    virtual WyRet sendbreak(int duration);
    virtual WyRet flush(int qsel);
    virtual WyRet flow(int action);
    virtual WyRet setattr(const WyTermios& tio,int option);
    WyRet getattr(WyTermios& tio) const;
    WyRet devname(WyStr& pathname) const;
    virtual WyRet drain(void);
};

#endif
